# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The LAMA Dataset"""


import json
from fnmatch import fnmatch

import datasets


_CITATION = """@inproceedings{petroni2019language,
  title={Language Models as Knowledge Bases?},
  author={F. Petroni, T. Rockt{\"{a}}schel, A. H. Miller, P. Lewis, A. Bakhtin, Y. Wu and S. Riedel},
  booktitle={In: Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing (EMNLP), 2019},
  year={2019}
}
@inproceedings{petroni2020how,
  title={How Context Affects Language Models' Factual Predictions},
  author={Fabio Petroni and Patrick Lewis and Aleksandra Piktus and Tim Rockt{\"a}schel and Yuxiang Wu and Alexander H. Miller and Sebastian Riedel},
  booktitle={Automated Knowledge Base Construction},
  year={2020},
  url={https://openreview.net/forum?id=025X0zPfn}
}
"""


_DESCRIPTION = """LAMA is a dataset used to probe and analyze the factual and commonsense knowledge contained in pretrained language models. See https://github.com/facebookresearch/LAMA.
"""

_HOMEPAGE = "https://github.com/facebookresearch/LAMA"

_LICENSE = "The Creative Commons Attribution-Noncommercial 4.0 International License. see https://github.com/facebookresearch/LAMA/blob/master/LICENSE"

_RELATIONS_URL = "https://s3.amazonaws.com/datasets.huggingface.co/lama/relations.jsonl"

_DATA_URL = "https://dl.fbaipublicfiles.com/LAMA/negated_data.tar.gz"


class Lama(datasets.GeneratorBasedBuilder):
    """Lama Dataset"""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="trex", version=VERSION, description="The TRex part of the Lama dataset"),
        datasets.BuilderConfig(name="squad", version=VERSION, description="The Squad part of the Lama dataset"),
        datasets.BuilderConfig(
            name="google_re", version=VERSION, description="The Google_re part of the Lama dataset"
        ),
        datasets.BuilderConfig(
            name="conceptnet", version=VERSION, description="The Conceptnet part of the Lama dataset"
        ),
    ]

    DEFAULT_CONFIG_NAME = "trex"

    def _info(self):
        if self.config.name == "trex":
            features = datasets.Features(
                {
                    "uuid": datasets.Value("string"),
                    "obj_uri": datasets.Value("string"),
                    "obj_label": datasets.Value("string"),
                    "sub_uri": datasets.Value("string"),
                    "sub_label": datasets.Value("string"),
                    "predicate_id": datasets.Value("string"),
                    "sub_surface": datasets.Value("string"),
                    "obj_surface": datasets.Value("string"),
                    "masked_sentence": datasets.Value("string"),
                    "template": datasets.Value("string"),
                    "template_negated": datasets.Value("string"),
                    "label": datasets.Value("string"),
                    "description": datasets.Value("string"),
                    "type": datasets.Value("string"),
                }
            )
            return datasets.DatasetInfo(
                description=_DESCRIPTION,
                features=features,
                supervised_keys=None,
                homepage=_HOMEPAGE,
                license=_LICENSE,
                citation=_CITATION,
            )
        elif self.config.name == "conceptnet":
            features = datasets.Features(
                {
                    "uuid": datasets.Value("string"),
                    "sub": datasets.Value("string"),
                    "obj": datasets.Value("string"),
                    "pred": datasets.Value("string"),
                    "obj_label": datasets.Value("string"),
                    "masked_sentence": datasets.Value("string"),
                    "negated": datasets.Value("string"),
                }
            )
            return datasets.DatasetInfo(
                description=_DESCRIPTION,
                features=features,
                supervised_keys=None,
                homepage=_HOMEPAGE,
                license=_LICENSE,
                citation=_CITATION,
            )
        elif self.config.name == "squad":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "sub_label": datasets.Value("string"),
                    "obj_label": datasets.Value("string"),
                    "negated": datasets.Value("string"),
                    "masked_sentence": datasets.Value("string"),
                }
            )
            return datasets.DatasetInfo(
                description=_DESCRIPTION,
                features=features,
                supervised_keys=None,
                homepage=_HOMEPAGE,
                license=_LICENSE,
                citation=_CITATION,
            )
        elif self.config.name == "google_re":
            features = datasets.Features(
                {
                    "pred": datasets.Value("string"),
                    "sub": datasets.Value("string"),
                    "obj": datasets.Value("string"),
                    "evidences": datasets.Value("string"),
                    "judgments": datasets.Value("string"),
                    "sub_w": datasets.Value("string"),
                    "sub_label": datasets.Value("string"),
                    "sub_aliases": datasets.Value("string"),
                    "obj_w": datasets.Value("string"),
                    "obj_label": datasets.Value("string"),
                    "obj_aliases": datasets.Value("string"),
                    "uuid": datasets.Value("string"),
                    "masked_sentence": datasets.Value("string"),
                    "template": datasets.Value("string"),
                    "template_negated": datasets.Value("string"),
                }
            )
            return datasets.DatasetInfo(
                description=_DESCRIPTION,
                features=features,
                supervised_keys=None,
                homepage=_HOMEPAGE,
                license=_LICENSE,
                citation=_CITATION,
            )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        archive = dl_manager.download(_DATA_URL)
        if self.config.name == "trex":
            relations_path = dl_manager.download(_RELATIONS_URL)
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "filepaths": ["TREx/*"],
                        "files": dl_manager.iter_archive(archive),
                        "relations_path": relations_path,
                    },
                ),
            ]
        elif self.config.name == "google_re":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "filepaths": [
                            "Google_RE/date_of_birth_test.jsonl",
                            "Google_RE/place_of_birth_test.jsonl",
                            "Google_RE/place_of_death_test.jsonl",
                        ],
                        "files": dl_manager.iter_archive(archive),
                    },
                ),
            ]
        elif self.config.name == "conceptnet":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "filepaths": ["ConceptNet/test.jsonl"],
                        "files": dl_manager.iter_archive(archive),
                    },
                ),
            ]
        elif self.config.name == "squad":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "filepaths": ["Squad/test.jsonl"],
                        "files": dl_manager.iter_archive(archive),
                    },
                ),
            ]

    def _generate_examples(self, filepaths, files, relations_path=None):
        """Yields examples from the LAMA dataset."""
        filepaths = list(filepaths)
        if self.config.name == "trex":
            all_rels = {}
            with open(relations_path, encoding="utf-8") as f:
                for row in f:
                    data = json.loads(row)
                    all_rels[data["relation"]] = data
            id_ = -1
            inside_trec_directory = False
            for path, f in files:
                if any(fnmatch(path, pattern) for pattern in filepaths):
                    inside_trec_directory = True
                    for row in f:
                        data = json.loads(row)
                        pred = all_rels.get(data["predicate_id"], {})
                        for evidences in data["evidences"]:
                            id_ += 1
                            yield id_, {
                                "uuid": str(data["uuid"]),
                                "obj_uri": str(data["obj_uri"]),
                                "obj_label": str(data["obj_label"]),
                                "sub_uri": str(data["sub_uri"]),
                                "sub_label": str(data["sub_label"]),
                                "predicate_id": str(data["predicate_id"]),
                                "sub_surface": str(evidences["sub_surface"]),
                                "obj_surface": str(evidences["obj_surface"]),
                                "masked_sentence": str(evidences["masked_sentence"]),
                                "template": str(pred.get("template", "")),
                                "template_negated": str(pred.get("template_negated", "")),
                                "label": str(pred.get("label", "")),
                                "description": str(pred.get("description", "")),
                                "type": str(pred.get("type", "")),
                            }
                elif inside_trec_directory:
                    break
        elif self.config.name == "conceptnet":
            id_ = -1
            for path, f in files:
                if not filepaths:
                    break
                if path in list(filepaths):
                    for row in f:
                        data = json.loads(row)
                        if data.get("negated") is not None:
                            for masked_sentence, negated in zip(data["masked_sentences"], data["negated"]):
                                id_ += 1
                                yield id_, {
                                    "uuid": str(data["uuid"]),
                                    "sub": str(data.get("sub", "")),
                                    "obj": str(data.get("obj", "")),
                                    "pred": str(data["pred"]),
                                    "obj_label": str(data["obj_label"]),
                                    "masked_sentence": str(masked_sentence),
                                    "negated": str(negated),
                                }
                        else:
                            for masked_sentence in data["masked_sentences"]:
                                id_ += 1
                                yield id_, {
                                    "uuid": str(data["uuid"]),
                                    "sub": str(data.get("sub", "")),
                                    "obj": str(data.get("obj", "")),
                                    "pred": str(data["pred"]),
                                    "obj_label": str(data["obj_label"]),
                                    "masked_sentence": str(masked_sentence),
                                    "negated": str(""),
                                }
                    filepaths.remove(path)
        elif self.config.name == "squad":
            id_ = -1
            for path, f in files:
                if not filepaths:
                    break
                if path in filepaths:
                    for row in f:
                        data = json.loads(row)
                        for masked_sentence in data["masked_sentences"]:
                            id_ += 1
                            yield id_, {
                                "id": str(data["id"]),
                                "sub_label": str(data["sub_label"]),
                                "obj_label": str(data["obj_label"]),
                                "negated": str(data.get("negated", "")),
                                "masked_sentence": str(masked_sentence),
                            }
                    filepaths.remove(path)
        elif self.config.name == "google_re":
            id_ = -1
            for path, f in files:
                if path in filepaths:
                    if not filepaths:
                        break
                    if path in filepaths:
                        # from https://github.com/facebookresearch/LAMA/blob/master/scripts/run_experiments.py
                        if "place_of_birth" in path:
                            pred = {
                                "relation": "place_of_birth",
                                "template": "[X] was born in [Y] .",
                                "template_negated": "[X] was not born in [Y] .",
                            }
                        elif "date_of_birth" in path:
                            pred = {
                                "relation": "date_of_birth",
                                "template": "[X] (born [Y]).",
                                "template_negated": "[X] (not born [Y]).",
                            }
                        else:
                            pred = {
                                "relation": "place_of_death",
                                "template": "[X] died in [Y] .",
                                "template_negated": "[X] did not die in [Y] .",
                            }
                        for row in f:
                            data = json.loads(row)
                            for masked_sentence in data["masked_sentences"]:
                                id_ += 1
                                yield id_, {
                                    "pred": str(data["pred"]),
                                    "sub": str(data["sub"]),
                                    "obj": str(data["obj"]),
                                    "evidences": str(data["evidences"]),
                                    "judgments": str(data["judgments"]),
                                    "sub_w": str(data["sub_w"]),
                                    "sub_label": str(data["sub_label"]),
                                    "sub_aliases": str(data["sub_aliases"]),
                                    "obj_w": str(data["obj_w"]),
                                    "obj_label": str(data["obj_label"]),
                                    "obj_aliases": str(data["obj_aliases"]),
                                    "uuid": str(data["uuid"]),
                                    "masked_sentence": str(masked_sentence),
                                    "template": str(pred["template"]),
                                    "template_negated": str(pred["template_negated"]),
                                }
                        filepaths.remove(path)



"""TODO(jeopardy): Add a description here."""


import json

import datasets


# TODO(jeopardy): BibTeX citation
_CITATION = """
"""

# TODO(jeopardy):
_DESCRIPTION = """
Dataset containing 216,930 Jeopardy questions, answers and other data.

The json file is an unordered list of questions where each question has
'category' : the question category, e.g. "HISTORY"
'value' : integer $ value of the question as string, e.g. "200"
Note: This is "None" for Final Jeopardy! and Tiebreaker questions
'question' : text of question
Note: This sometimes contains hyperlinks and other things messy text such as when there's a picture or video question
'answer' : text of answer
'round' : one of "Jeopardy!","Double Jeopardy!","Final Jeopardy!" or "Tiebreaker"
Note: Tiebreaker questions do happen but they're very rare (like once every 20 years)
'show_number' : int of show number, e.g '4680'
'air_date' : string of the show air date in format YYYY-MM-DD
"""
_URL = "https://www.reddit.com/r/datasets/comments/1uyd0t/200000_jeopardy_questions_in_a_json_file/"
_DATA_URL = "https://drive.google.com/uc?export=download&id=0BwT5wj_P7BKXb2hfM3d2RHU1ckE"
_DATA_FILE = "JEOPARDY_QUESTIONS1.json"


class Jeopardy(datasets.GeneratorBasedBuilder):
    """TODO(jeopardy): Short description of my dataset."""

    # TODO(jeopardy): Set up version.
    VERSION = datasets.Version("0.1.0")

    def _info(self):
        # TODO(jeopardy): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "category": datasets.Value("string"),
                    "air_date": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "value": datasets.Value("int32"),
                    "answer": datasets.Value("string"),
                    "round": datasets.Value("string"),
                    "show_number": datasets.Value("int32"),
                    # These are the features of your dataset like images, labels ...
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(jeopardy): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        filepath = dl_manager.download_and_extract(_DATA_URL)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": filepath}),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        # TODO(jeopardy): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            data = json.load(f)
            for i, example in enumerate(data):
                category = example["category"]
                air_date = example["air_date"]
                question = example["question"]
                if example["value"] is None:
                    value = -1  # for Final Jeopardy! and Tiebreaker questions
                else:
                    value = int(example["value"][1:].replace(",", ""))
                answer = example["answer"]
                round = example["round"]
                show_number = int(example["show_number"])
                yield i, {
                    "category": category,
                    "air_date": air_date,
                    "question": question,
                    "value": value,
                    "answer": answer,
                    "round": round,
                    "category": category,
                    "show_number": show_number,
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""C3 Parallel Corpora"""


import json

import datasets


_CITATION = """\
@article{sun2019investigating,
  title={Investigating Prior Knowledge for Challenging Chinese Machine Reading Comprehension},
  author={Sun, Kai and Yu, Dian and Yu, Dong and Cardie, Claire},
  journal={Transactions of the Association for Computational Linguistics},
  year={2020},
  url={https://arxiv.org/abs/1904.09679v3}
}
"""

_DESCRIPTION = """\
Machine reading comprehension tasks require a machine reader to answer questions relevant to the given document. In this paper, we present the first free-form multiple-Choice Chinese machine reading Comprehension dataset (C^3), containing 13,369 documents (dialogues or more formally written mixed-genre texts) and their associated 19,577 multiple-choice free-form questions collected from Chinese-as-a-second-language examinations.
We present a comprehensive analysis of the prior knowledge (i.e., linguistic, domain-specific, and general world knowledge) needed for these real-world problems. We implement rule-based and popular neural methods and find that there is still a significant performance gap between the best performing model (68.5%) and human readers (96.0%), especially on problems that require prior knowledge. We further study the effects of distractor plausibility and data augmentation based on translated relevant datasets for English on model performance. We expect C^3 to present great challenges to existing systems as answering 86.8% of questions requires both knowledge within and beyond the accompanying document, and we hope that C^3 can serve as a platform to study how to leverage various kinds of prior knowledge to better understand a given written or orally oriented text.
"""

_URL = "https://raw.githubusercontent.com/nlpdata/c3/master/data/"


class C3Config(datasets.BuilderConfig):
    """BuilderConfig for NewDataset"""

    def __init__(self, type_, **kwargs):
        """

        Args:
            pair: the language pair to consider
            zip_file: The location of zip file containing original data
            **kwargs: keyword arguments forwarded to super.
        """
        self.type_ = type_
        super().__init__(**kwargs)


class C3(datasets.GeneratorBasedBuilder):
    """C3 is the first free-form multiple-Choice Chinese machine reading Comprehension dataset, containing 13,369 documents (dialogues or more formally written mixed-genre texts) and their associated 19,577 multiple-choice free-form questions collected from Chinese-as-a-second language examinations."""

    VERSION = datasets.Version("1.0.0")

    # This is an example of a dataset with multiple configurations.
    # If you don't want/need to define several sub-sets in your dataset,
    # just remove the BUILDER_CONFIG_CLASS and the BUILDER_CONFIGS attributes.
    BUILDER_CONFIG_CLASS = C3Config
    BUILDER_CONFIGS = [
        C3Config(
            name="mixed",
            description="Mixed genre questions",
            version=datasets.Version("1.0.0"),
            type_="mixed",
        ),
        C3Config(
            name="dialog",
            description="Dialog questions",
            version=datasets.Version("1.0.0"),
            type_="dialog",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "documents": datasets.Sequence(datasets.Value("string")),
                    "document_id": datasets.Value("string"),
                    "questions": datasets.Sequence(
                        {
                            "question": datasets.Value("string"),
                            "answer": datasets.Value("string"),
                            "choice": datasets.Sequence(datasets.Value("string")),
                        }
                    ),
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://github.com/nlpdata/c3",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        # m or d
        T = self.config.type_[0]
        files = [_URL + f"c3-{T}-{split}.json" for split in ["train", "test", "dev"]]
        dl_dir = dl_manager.download_and_extract(files)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filename": dl_dir[0],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filename": dl_dir[1],
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filename": dl_dir[2],
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filename, split):
        """Yields examples."""
        with open(filename, "r", encoding="utf-8") as sf:
            data = json.load(sf)
            for id_, (documents, questions, document_id) in enumerate(data):
                yield id_, {
                    "documents": documents,
                    "questions": questions,
                    "document_id": document_id,
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Wikicorpus dataset."""

import re

import datasets


_CITATION = """\
@inproceedings{reese-etal-2010-wikicorpus,
    title = "{W}ikicorpus: A Word-Sense Disambiguated Multilingual {W}ikipedia Corpus",
    author = "Reese, Samuel  and
      Boleda, Gemma  and
      Cuadros, Montse  and
      Padr{\'o}, Llu{\'i}s  and
      Rigau, German",
    booktitle = "Proceedings of the Seventh International Conference on Language Resources and Evaluation ({LREC}'10)",
    month = may,
    year = "2010",
    address = "Valletta, Malta",
    publisher = "European Language Resources Association (ELRA)",
    url = "http://www.lrec-conf.org/proceedings/lrec2010/pdf/222_Paper.pdf",
    abstract = "This article presents a new freely available trilingual corpus (Catalan, Spanish, English) that contains large portions of the Wikipedia and has been automatically enriched with linguistic information. To our knowledge, this is the largest such corpus that is freely available to the community: In its present version, it contains over 750 million words. The corpora have been annotated with lemma and part of speech information using the open source library FreeLing. Also, they have been sense annotated with the state of the art Word Sense Disambiguation algorithm UKB. As UKB assigns WordNet senses, and WordNet has been aligned across languages via the InterLingual Index, this sort of annotation opens the way to massive explorations in lexical semantics that were not possible before. We present a first attempt at creating a trilingual lexical resource from the sense-tagged Wikipedia corpora, namely, WikiNet. Moreover, we present two by-products of the project that are of use for the NLP community: An open source Java-based parser for Wikipedia pages developed for the construction of the corpus, and the integration of the WSD algorithm UKB in FreeLing.",
}
"""

_DESCRIPTION = """\
The Wikicorpus is a trilingual corpus (Catalan, Spanish, English) that contains large portions of the Wikipedia (based on a 2006 dump) and has been automatically enriched with linguistic information. In its present version, it contains over 750 million words.
"""

_HOMEPAGE = "https://www.cs.upc.edu/~nlp/wikicorpus/"

_LICENSE = "GNU Free Documentation License"

_URLs = "https://www.cs.upc.edu/~nlp/wikicorpus/{form}.{language}.tgz"

_LANGUAGES = ["ca", "es", "en"]
_FORMS = ["raw", "tagged"]

METADATA_PATTERN = re.compile(r'.+id="(?P<id>[^"]+)".+title="(?P<title>[^"]+)".+')


class WikicorpusConfig(datasets.BuilderConfig):
    """BuilderConfig for Wikicorpus."""

    def __init__(self, form=None, language=None, **kwargs):
        """
        Args:
            form: form of the dataset.
            language: language of the dataset.
            **kwargs: keyword arguments forwarded to super.
        """
        super().__init__(
            name=f"{form}_{language}",
            description=f"Wikicorpus dataset in {form} form and {language} language.",
            **kwargs,
        )
        self.form = form
        self.language = language


class Wikicorpus(datasets.GeneratorBasedBuilder):
    """Wikicorpus dataset."""

    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIG_CLASS = WikicorpusConfig
    BUILDER_CONFIGS = [WikicorpusConfig(form=form, language=language) for form in _FORMS for language in _LANGUAGES]

    def _info(self):
        if self.config.form == "raw":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.form == "tagged":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "sentence": datasets.Sequence(datasets.Value("string")),
                    "lemmas": datasets.Sequence(datasets.Value("string")),
                    "pos_tags": datasets.Sequence(datasets.Value("string")),
                    "wordnet_senses": datasets.Sequence(datasets.Value("string")),
                }
            )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        url_to_download = _URLs.format(form=self.config.form, language=self.config.language)
        archive = dl_manager.download(url_to_download)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "files": dl_manager.iter_archive(archive),
                },
            ),
        ]

    def _generate_examples(self, files):
        for file_idx, (path, f) in enumerate(files):
            example = {}
            # raw
            text = []
            # tagged
            words = []
            lemmas = []
            pos_tags = []
            wordnet_senses = []
            for row_idx, row in enumerate(f):
                row = row.decode("latin-1")
                if self.config.form == "raw":
                    if row.startswith("<doc id"):
                        metadata_match = METADATA_PATTERN.match(row)
                        example["id"] = metadata_match.group("id") if metadata_match else ""
                        example["title"] = metadata_match.group("title") if metadata_match else ""
                    elif row.startswith("</doc>"):
                        pass
                    elif row.startswith("ENDOFARTICLE"):
                        yield f"{file_idx}_{row_idx}", {
                            "id": example["id"],
                            "title": example["title"],
                            "text": "\n".join(text).strip(),
                        }
                        example = {}
                        text = []
                    else:
                        text.append(row)
                elif self.config.form == "tagged":
                    if row.startswith("<doc id"):
                        metadata_match = METADATA_PATTERN.match(row)
                        example["id"] = metadata_match.group("id") if metadata_match else ""
                        example["title"] = metadata_match.group("title") if metadata_match else ""
                    elif row.startswith("</doc>"):
                        pass
                    elif row.startswith("ENDOFARTICLE") or row.startswith("\n"):
                        if len(words) > 1:  # some content besides only (. . Fp 0)
                            yield f"{file_idx}_{row_idx}", {
                                "id": example["id"],
                                "title": example["title"],
                                "sentence": words,
                                "lemmas": lemmas,
                                "pos_tags": pos_tags,
                                "wordnet_senses": wordnet_senses,
                            }
                        words = []
                        lemmas = []
                        pos_tags = []
                        wordnet_senses = []
                        if row.startswith("ENDOFARTICLE"):
                            example = {}
                    else:
                        splits = row.split()
                        for tag, tags in zip(splits, [words, lemmas, pos_tags, wordnet_senses]):
                            tags.append(tag)



"""TODO(squad_v1_pt): Add a description here."""


import json

import datasets
from datasets.tasks import QuestionAnsweringExtractive


# TODO(squad_v1_pt): BibTeX citation
_CITATION = """\
@article{2016arXiv160605250R,
       author = {{Rajpurkar}, Pranav and {Zhang}, Jian and {Lopyrev},
                 Konstantin and {Liang}, Percy},
        title = "{SQuAD: 100,000+ Questions for Machine Comprehension of Text}",
      journal = {arXiv e-prints},
         year = 2016,
          eid = {arXiv:1606.05250},
        pages = {arXiv:1606.05250},
archivePrefix = {arXiv},
       eprint = {1606.05250},
}
"""

# TODO(squad_v1_pt):
_DESCRIPTION = """\
Portuguese translation of the SQuAD dataset. The translation was performed automatically using the Google Cloud API.
"""

_URL = "https://github.com/nunorc/squad-v1.1-pt/raw/master/"
_URLS = {
    "train": _URL + "train-v1.1-pt.json",
    "dev": _URL + "dev-v1.1-pt.json",
}


class SquadV1Pt(datasets.GeneratorBasedBuilder):
    """TODO(squad_v1_pt): Short description of my dataset."""

    # TODO(squad_v1_pt): Set up version.
    VERSION = datasets.Version("1.1.0")

    def _info(self):
        # TODO(squad_v1_pt): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "context": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "answers": datasets.features.Sequence(
                        {
                            "text": datasets.Value("string"),
                            "answer_start": datasets.Value("int32"),
                        }
                    ),
                    # These are the features of your dataset like images, labels ...
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://github.com/nunorc/squad-v1.1-pt",
            citation=_CITATION,
            task_templates=[
                QuestionAnsweringExtractive(
                    question_column="question", context_column="context", answers_column="answers"
                )
            ],
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(squad_v1_pt): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        urls_to_download = _URLS
        downloaded_files = dl_manager.download_and_extract(urls_to_download)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["train"]}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": downloaded_files["dev"]}),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        # TODO(squad_v1_pt): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            data = json.load(f)
            for example in data["data"]:
                title = example.get("title", "").strip()
                for paragraph in example["paragraphs"]:
                    context = paragraph["context"].strip()
                    for qa in paragraph["qas"]:
                        question = qa["question"].strip()
                        id_ = qa["id"]

                        answer_starts = [answer["answer_start"] for answer in qa["answers"]]
                        answers = [answer["text"].strip() for answer in qa["answers"]]

                        yield id_, {
                            "title": title,
                            "context": context,
                            "question": question,
                            "id": id_,
                            "answers": {
                                "answer_start": answer_starts,
                                "text": answers,
                            },
                        }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Enriched WebNLG corpus"""


import itertools
import os
import xml.etree.cElementTree as ET
from collections import defaultdict
from glob import glob
from os.path import join as pjoin

import datasets


_CITATION = """\
@InProceedings{ferreiraetal2018,
  author = 	"Castro Ferreira, Thiago and Moussallem, Diego and Wubben, Sander and Krahmer, Emiel",
  title = 	"Enriching the WebNLG corpus",
  booktitle = 	"Proceedings of the 11th International Conference on Natural Language Generation",
  year = 	"2018",
  series = {INLG'18},
  publisher = 	"Association for Computational Linguistics",
  address = 	"Tilburg, The Netherlands",
}
"""

_DESCRIPTION = """\
WebNLG is a valuable resource and benchmark for the Natural Language Generation (NLG) community. However, as other NLG benchmarks, it only consists of a collection of parallel raw representations and their corresponding textual realizations. This work aimed to provide intermediate representations of the data for the development and evaluation of popular tasks in the NLG pipeline architecture (Reiter and Dale, 2000), such as Discourse Ordering, Lexicalization, Aggregation and Referring Expression Generation.
"""

_HOMEPAGE = "https://github.com/ThiagoCF05/webnlg"

_LICENSE = "CC Attribution-Noncommercial-Share Alike 4.0 International"

_SHA = "12ca34880b225ebd1eb9db07c64e8dd76f7e5784"

_URL = f"https://github.com/ThiagoCF05/webnlg/archive/{_SHA}.zip"

_FILE_PATHS = {
    "en": {
        "train": [f"webnlg-{_SHA}/data/v1.5/en/train/{i}triples/" for i in range(1, 8)],
        "dev": [f"webnlg-{_SHA}/data/v1.5/en/dev/{i}triples/" for i in range(1, 8)],
        "test": [f"webnlg-{_SHA}/data/v1.5/en/test/{i}triples/" for i in range(1, 8)],
    },
    "de": {
        "train": [f"webnlg-{_SHA}/data/v1.5/de/train/{i}triples/" for i in range(1, 8)],
        "dev": [f"webnlg-{_SHA}/data/v1.5/de/dev/{i}triples/" for i in range(1, 8)],
    },
}


def et_to_dict(tree):
    """Takes the xml tree within a dataset file and returns a dictionary with entry data"""
    dct = {tree.tag: {} if tree.attrib else None}
    children = list(tree)
    if children:
        dd = defaultdict(list)
        for dc in map(et_to_dict, children):
            for k, v in dc.items():
                dd[k].append(v)
        dct = {tree.tag: dd}
    if tree.attrib:
        dct[tree.tag].update((k, v) for k, v in tree.attrib.items())
    if tree.text:
        text = tree.text.strip()
        if children or tree.attrib:
            if text:
                dct[tree.tag]["text"] = text
        else:
            dct[tree.tag] = text
    return dct


def parse_entry(entry, config_name):
    """Takes the dictionary corresponding to an entry and returns a dictionary with:
    - Proper feature naming
    - Default values
    - Proper typing"""
    res = {}
    otriple_set_list = entry["originaltripleset"]
    res["original_triple_sets"] = [{"otriple_set": otriple_set["otriple"]} for otriple_set in otriple_set_list]
    mtriple_set_list = entry["modifiedtripleset"]
    res["modified_triple_sets"] = [{"mtriple_set": mtriple_set["mtriple"]} for mtriple_set in mtriple_set_list]
    res["category"] = entry["category"]
    res["eid"] = entry["eid"]
    res["size"] = int(entry["size"])
    lex = entry["lex"]
    # Some entries are misformed, with None instead of the sorted triplet information.
    entry_triples = [
        ex["sortedtripleset"][0] if ex["sortedtripleset"][0] is not None else {"sentence": []} for ex in lex
    ]
    # the xml structure is inconsistent; sorted triplets are often separated in several dictionaries, so we sum them.
    sorted_triples = [
        list(itertools.chain.from_iterable(item.get("striple", []) for item in entry["sentence"]))
        for entry in entry_triples
    ]
    res["lex"] = {
        "comment": [ex.get("comment", "") for ex in lex],
        "lid": [ex.get("lid", "") for ex in lex],
        # all of the sequence are within their own 1-element sublist, thus the [0]
        "text": [ex.get("text", [""])[0] for ex in lex],
        "template": [ex.get("template", [""])[0] for ex in lex],
        "sorted_triple_sets": sorted_triples,
    }
    # only present in the en version
    if config_name == "en":
        res["lex"]["lexicalization"] = [ex.get("lexicalization", [""])[0] for ex in lex]
    res["shape"] = entry.get("shape", "")
    res["shape_type"] = entry.get("shape_type", "")
    return res


def xml_file_to_examples(filename, config_name):
    tree = ET.parse(filename).getroot()
    examples = et_to_dict(tree)["benchmark"]["entries"][0]["entry"]
    return [parse_entry(entry, config_name) for entry in examples]


class EnrichedWebNlg(datasets.GeneratorBasedBuilder):
    """The WebNLG corpus"""

    VERSION = datasets.Version("1.5.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="en", description="Enriched English version of the WebNLG data"),
        datasets.BuilderConfig(name="de", description="Enriched German version of the WebNLG data"),
    ]

    def _info(self):
        if self.config.name == "en":
            features = datasets.Features(
                {
                    "category": datasets.Value("string"),
                    "size": datasets.Value("int32"),
                    "eid": datasets.Value("string"),
                    "original_triple_sets": datasets.Sequence(
                        {"otriple_set": datasets.Sequence(datasets.Value("string"))}
                    ),
                    "modified_triple_sets": datasets.Sequence(
                        {"mtriple_set": datasets.Sequence(datasets.Value("string"))}
                    ),
                    "shape": datasets.Value("string"),
                    "shape_type": datasets.Value("string"),
                    "lex": datasets.Sequence(
                        {
                            "comment": datasets.Value("string"),
                            "lid": datasets.Value("string"),
                            "text": datasets.Value("string"),
                            "template": datasets.Value("string"),
                            "sorted_triple_sets": datasets.Sequence(datasets.Value("string")),
                            # only present in the en version
                            "lexicalization": datasets.Value("string"),
                        }
                    ),
                }
            )
        else:
            features = datasets.Features(
                {
                    "category": datasets.Value("string"),
                    "size": datasets.Value("int32"),
                    "eid": datasets.Value("string"),
                    "original_triple_sets": datasets.Sequence(
                        {"otriple_set": datasets.Sequence(datasets.Value("string"))}
                    ),
                    "modified_triple_sets": datasets.Sequence(
                        {"mtriple_set": datasets.Sequence(datasets.Value("string"))}
                    ),
                    "shape": datasets.Value("string"),
                    "shape_type": datasets.Value("string"),
                    "lex": datasets.Sequence(
                        {
                            "comment": datasets.Value("string"),
                            "lid": datasets.Value("string"),
                            "text": datasets.Value("string"),
                            "template": datasets.Value("string"),
                            "sorted_triple_sets": datasets.Sequence(datasets.Value("string")),
                        }
                    ),
                }
            )
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=features,  # Here we define them above because they are different between the two configurations
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            citation=_CITATION,
            license=_LICENSE,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_URL)
        # Downloading the repo adds the current commit sha to the directory, so we can't specify the directory
        # name in advance.
        split_files = {
            split: [os.path.join(data_dir, dir_suf) for dir_suf in dir_suffix_list]
            for split, dir_suffix_list in _FILE_PATHS[self.config.name].items()
        }
        return [
            datasets.SplitGenerator(
                name=split,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filedirs": filedirs},
            )
            for split, filedirs in split_files.items()
        ]

    def _generate_examples(self, filedirs):
        """Yields examples."""

        id_ = 0
        for xml_location in filedirs:
            for xml_file in sorted(glob(pjoin(xml_location, "*.xml"))):
                for exple_dict in xml_file_to_examples(xml_file, self.config.name):
                    id_ += 1
                    yield id_, exple_dict



"""`prachathai-67k`: News Article Corpus and Multi-label Text Classificdation from Prachathai.com"""


import json
import os

import datasets


_CITATION = """\
@misc{prachathai67k,
  author = {cstorm125, lukkiddd },
  title = {prachathai67k},
  year = {2019},
  publisher = {GitHub},
  journal = {GitHub repository},
  howpublished={\\url{https://github.com/PyThaiNLP/prachathai-67k}},
}
"""

_DESCRIPTION = """\
`prachathai-67k`: News Article Corpus and Multi-label Text Classificdation from Prachathai.com
The prachathai-67k dataset was scraped from the news site Prachathai.
We filtered out those articles with less than 500 characters of body text, mostly images and cartoons.
It contains 67,889 articles wtih 12 curated tags from August 24, 2004 to November 15, 2018.
The dataset was originally scraped by @lukkiddd and cleaned by @cstorm125.
You can also see preliminary exploration at https://github.com/PyThaiNLP/prachathai-67k/blob/master/exploration.ipynb
"""


class Prachathai67kConfig(datasets.BuilderConfig):
    """BuilderConfig for Prachathai."""

    def __init__(self, **kwargs):
        """BuilderConfig for Prachathai.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(Prachathai67kConfig, self).__init__(**kwargs)


class Prachathai67k(datasets.GeneratorBasedBuilder):
    """`prachathai-67k`: News Article Corpus and Multi-label Text Classificdation from Prachathai.com"""

    _DOWNLOAD_URL = "https://archive.org/download/prachathai67k/data.zip"
    _TRAIN_FILE = "train.jsonl"
    _VAL_FILE = "valid.jsonl"
    _TEST_FILE = "test.jsonl"

    BUILDER_CONFIGS = [
        Prachathai67kConfig(
            name="prachathai67k",
            version=datasets.Version("1.1.0"),
            description="`prachathai-67k`: News Article Corpus and Multi-label Text Classificdation from Prachathai.com",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "url": datasets.Value("string"),
                    "date": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "body_text": datasets.Value("string"),
                    "politics": datasets.features.ClassLabel(names=["neg", "pos"]),
                    "human_rights": datasets.features.ClassLabel(names=["neg", "pos"]),
                    "quality_of_life": datasets.features.ClassLabel(names=["neg", "pos"]),
                    "international": datasets.features.ClassLabel(names=["neg", "pos"]),
                    "social": datasets.features.ClassLabel(names=["neg", "pos"]),
                    "environment": datasets.features.ClassLabel(names=["neg", "pos"]),
                    "economics": datasets.features.ClassLabel(names=["neg", "pos"]),
                    "culture": datasets.features.ClassLabel(names=["neg", "pos"]),
                    "labor": datasets.features.ClassLabel(names=["neg", "pos"]),
                    "national_security": datasets.features.ClassLabel(names=["neg", "pos"]),
                    "ict": datasets.features.ClassLabel(names=["neg", "pos"]),
                    "education": datasets.features.ClassLabel(names=["neg", "pos"]),
                }
            ),
            supervised_keys=None,
            homepage="https://github.com/PyThaiNLP/prachathai-67k/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        arch_path = dl_manager.download_and_extract(self._DOWNLOAD_URL)
        data_dir = os.path.join(arch_path, "data")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": os.path.join(data_dir, self._TRAIN_FILE)},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": os.path.join(data_dir, self._VAL_FILE)},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(data_dir, self._TEST_FILE)},
            ),
        ]

    def _generate_examples(self, filepath):
        """Generate examples."""
        with open(filepath, encoding="utf-8") as f:
            for id_, row in enumerate(f):
                data = json.loads(row)
                yield id_, {
                    "url": data["url"],
                    "date": data["date"],
                    "title": data["title"],
                    "body_text": data["body_text"],
                    "politics": data["politics"],
                    "human_rights": data["human_rights"],
                    "quality_of_life": data["quality_of_life"],
                    "international": data["international"],
                    "social": data["social"],
                    "environment": data["environment"],
                    "economics": data["economics"],
                    "culture": data["culture"],
                    "labor": data["labor"],
                    "national_security": data["national_security"],
                    "ict": data["ict"],
                    "education": data["education"],
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""WinoBias: Winograd-schema dataset for detecting gender bias"""


import collections

import datasets


_CITATION = """\
@article{DBLP:journals/corr/abs-1804-06876,
  author    = {Jieyu Zhao and
               Tianlu Wang and
               Mark Yatskar and
               Vicente Ordonez and
               Kai{-}Wei Chang},
  title     = {Gender Bias in Coreference Resolution: Evaluation and Debiasing Methods},
  journal   = {CoRR},
  volume    = {abs/1804.06876},
  year      = {2018},
  url       = {http://arxiv.org/abs/1804.06876},
  archivePrefix = {arXiv},
  eprint    = {1804.06876},
  timestamp = {Mon, 13 Aug 2018 16:47:01 +0200},
  biburl    = {https://dblp.org/rec/journals/corr/abs-1804-06876.bib},
  bibsource = {dblp computer science bibliography, https://dblp.org}
}
"""

_DESCRIPTION = """\
WinoBias, a Winograd-schema dataset for coreference resolution focused on gender bias.
The corpus contains Winograd-schema style sentences with entities corresponding to people
referred by their occupation (e.g. the nurse, the doctor, the carpenter).
"""

_HOMEPAGE = "https://uclanlp.github.io/corefBias/overview"

_LICENSE = "MIT License (https://github.com/uclanlp/corefBias/blob/master/LICENSE)"

_URL = "https://raw.githubusercontent.com/uclanlp/corefBias/master/WinoBias/wino/data/conll_format"


class WinoBiasConfig(datasets.BuilderConfig):
    def __init__(self, **kwargs):
        super(WinoBiasConfig, self).__init__(version=datasets.Version("1.0.0", ""), **kwargs)


class WinoBias(datasets.GeneratorBasedBuilder):
    """WinoBias: Winograd-schema dataset for detecting gender bias"""

    # This is an example of a dataset with multiple configurations.
    # If you don't want/need to define several sub-sets in your dataset,
    # just remove the BUILDER_CONFIG_CLASS and the BUILDER_CONFIGS attributes.

    # If you need to make complex sub-parts in the datasets with configurable options
    # You can create your own builder configuration class to store attribute, inheriting from datasets.BuilderConfig
    # BUILDER_CONFIG_CLASS = MyBuilderConfig

    # You will be able to load one or the other configurations in the following list with
    # data = datasets.load_dataset('my_dataset', 'first_domain')
    # data = datasets.load_dataset('my_dataset', 'second_domain')
    def __init__(self, *args, writer_batch_size=None, **kwargs):
        super(WinoBias, self).__init__(*args, **kwargs)
        # Batch size used by the ArrowWriter
        # It defines the number of samples that are kept in memory before writing them
        # and also the length of the arrow chunks
        # None means that the ArrowWriter will use its default value
        self._writer_batch_size = writer_batch_size or 100

    BUILDER_CONFIGS = [
        WinoBiasConfig(
            name="type1_pro",
            description="winoBias type1_pro_stereotype data in cornll format",
        ),
        WinoBiasConfig(
            name="type1_anti",
            description="winoBias type1_anti_stereotype data in cornll format",
        ),
        WinoBiasConfig(
            name="type2_pro",
            description="winoBias type2_pro_stereotype data in cornll format",
        ),
        WinoBiasConfig(
            name="type2_anti",
            description="winoBias type2_anti_stereotype data in cornll format",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            # Info about features for this: http://cemantix.org/data/ontonotes.html
            features=datasets.Features(
                {
                    "document_id": datasets.Value("string"),
                    "part_number": datasets.Value("string"),
                    "word_number": datasets.Sequence(datasets.Value("int32")),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "pos_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                '"',
                                "''",
                                "#",
                                "$",
                                "(",
                                ")",
                                ",",
                                ".",
                                ":",
                                "``",
                                "CC",
                                "CD",
                                "DT",
                                "EX",
                                "FW",
                                "IN",
                                "JJ",
                                "JJR",
                                "JJS",
                                "LS",
                                "MD",
                                "NN",
                                "NNP",
                                "NNPS",
                                "NNS",
                                "NN|SYM",
                                "PDT",
                                "POS",
                                "PRP",
                                "PRP$",
                                "RB",
                                "RBR",
                                "RBS",
                                "RP",
                                "SYM",
                                "TO",
                                "UH",
                                "VB",
                                "VBD",
                                "VBG",
                                "VBN",
                                "VBP",
                                "VBZ",
                                "WDT",
                                "WP",
                                "WP$",
                                "WRB",
                                "HYPH",
                                "XX",
                                "NFP",
                                "AFX",
                                "ADD",
                                "-LRB-",
                                "-RRB-",
                                "-",
                            ]
                        )
                    ),
                    "parse_bit": datasets.Sequence(datasets.Value("string")),
                    "predicate_lemma": datasets.Sequence(datasets.Value("string")),
                    "predicate_framenet_id": datasets.Sequence(datasets.Value("string")),
                    "word_sense": datasets.Sequence(datasets.Value("string")),
                    "speaker": datasets.Sequence(datasets.Value("string")),
                    "ner_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "B-PERSON",
                                "I-PERSON",
                                "B-NORP",
                                "I-NORP",
                                "B-FAC",
                                "I-FAC",
                                "B-ORG",
                                "I-ORG",
                                "B-GPE",
                                "I-GPE",
                                "B-LOC",
                                "I-LOC",
                                "B-PRODUCT",
                                "I-PRODUCT",
                                "B-EVENT",
                                "I-EVENT",
                                "B-WORK_OF_ART",
                                "I-WORK_OF_ART",
                                "B-LAW",
                                "I-LAW",
                                "B-LANGUAGE",
                                "I-LANGUAGE",
                                "B-DATE",
                                "I-DATE",
                                "B-TIME",
                                "I-TIME",
                                "B-PERCENT",
                                "I-PERCENT",
                                "B-MONEY",
                                "I-MONEY",
                                "B-QUANTITY",
                                "I-QUANTITY",
                                "B-ORDINAL",
                                "I-ORDINAL",
                                "B-CARDINAL",
                                "I-CARDINAL",
                                "*",
                                "0",
                                "-",
                            ]
                        )
                    ),
                    "verbal_predicates": datasets.Sequence(datasets.Value("string")),
                    "coreference_clusters": datasets.Sequence(datasets.Value("string")),
                }
            ),
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        dev_data_dir = dl_manager.download(_URL + "/dev_" + self.config.name + "_stereotype.v4_auto_conll")
        test_data_dir = dl_manager.download(_URL + "/test_" + self.config.name + "_stereotype.v4_auto_conll")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": dev_data_dir},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": test_data_dir},
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            id_ = 0
            document_id = None
            part_number = 0
            word_num = []
            tokens = []
            pos_tags = []
            parse_bit = []
            predicate_lemma = []
            predicate_framenet_id = []
            word_sense = []
            speaker = []
            ner_tags = []
            ner_start = False
            verbal_predicates = []
            coreference = []
            clusters = collections.defaultdict(list)
            coref_stacks = collections.defaultdict(list)
            for line in f:
                if line.startswith("#begin") or line.startswith("#end"):
                    continue
                elif not line.strip():
                    id_ += 1
                    yield str(id_), {
                        "document_id": document_id,
                        "part_number": part_number,
                        "word_number": word_num,
                        "tokens": tokens,
                        "pos_tags": pos_tags,
                        "parse_bit": parse_bit,
                        "predicate_lemma": predicate_lemma,
                        "predicate_framenet_id": predicate_framenet_id,
                        "word_sense": word_sense,
                        "speaker": speaker,
                        "ner_tags": ner_tags,
                        "verbal_predicates": verbal_predicates,
                        "coreference_clusters": sum(
                            clusters[1], []
                        ),  # flatten the list as writing the exmaples needs an array.
                    }

                    word_num = []
                    tokens = []
                    pos_tags = []
                    parse_bit = []
                    predicate_lemma = []
                    predicate_framenet_id = []
                    word_sense = []
                    speaker = []
                    ner_tags = []
                    verbal_predicates = []
                    coreference = []
                    clusters = collections.defaultdict(list)
                    coref_stacks = collections.defaultdict(list)
                else:
                    splits = [s for s in line.split() if s]
                    if len(splits) > 7:
                        document_id = splits[0]
                        part_number = splits[1]
                        word_num.append(splits[2])
                        tokens.append(splits[3])
                        pos_tags.append(splits[4])
                        parse_bit.append(splits[5])
                        predicate_lemma.append(splits[6])
                        predicate_framenet_id.append(splits[7])
                        word_sense.append(splits[8])
                        speaker.append(splits[9])
                        ner_word = splits[10]
                        coreference = splits[-1]
                        if ")" in ner_word and ner_start:
                            ner_start = False
                            ner_word = "0"
                        if "(" in ner_word:
                            ner_start = True
                            ner_word = ner_word.strip(" ").replace("(", "B-").replace("*", "").replace(")", "")
                            start_word = ner_word.strip(" ").replace("B-", "")
                        if ner_start:
                            if ner_word.strip(" ") == "*":
                                ner_word = "I-" + start_word
                        ner_tags.append(ner_word)
                        word_is_verbal_predicate = any(["(V" in x for x in splits[11:-1]])
                        if word_is_verbal_predicate:
                            verbal_predicates.append(splits[3])
                        if coreference != "-":
                            for segment in coreference.split("|"):
                                if segment[0] == "(":
                                    if segment[-1] == ")":
                                        cluster_id = int(segment[1:-1])
                                        clusters[cluster_id].append([splits[2], splits[2]])
                                    else:
                                        cluster_id = int(segment[1:])
                                        coref_stacks[cluster_id].append(splits[2])
                                else:
                                    cluster_id = int(segment[:-1])
                                    start = coref_stacks[cluster_id].pop()
                                    clusters[cluster_id].append([start, splits[2]])

            if tokens:
                # add the last one
                id_ += 1
                yield str(id_), {
                    "document_id": document_id,
                    "part_number": part_number,
                    "word_number": word_num,
                    "tokens": tokens,
                    "pos_tags": pos_tags,
                    "parse_bit": parse_bit,
                    "predicate_lemma": predicate_lemma,
                    "predicate_framenet_id": predicate_framenet_id,
                    "word_sense": word_sense,
                    "speaker": speaker,
                    "ner_tags": ner_tags,
                    "verbal_predicates": verbal_predicates,
                    "coreference_clusters": sum(clusters[1], []),
                }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""e-SNLI: Natural Language Inference with Natural Language Explanations."""


import csv

import datasets


_CITATION = """
@incollection{NIPS2018_8163,
title = {e-SNLI: Natural Language Inference with Natural Language Explanations},
author = {Camburu, Oana-Maria and Rockt\"{a}schel, Tim and Lukasiewicz, Thomas and Blunsom, Phil},
booktitle = {Advances in Neural Information Processing Systems 31},
editor = {S. Bengio and H. Wallach and H. Larochelle and K. Grauman and N. Cesa-Bianchi and R. Garnett},
pages = {9539--9549},
year = {2018},
publisher = {Curran Associates, Inc.},
url = {http://papers.nips.cc/paper/8163-e-snli-natural-language-inference-with-natural-language-explanations.pdf}
}
"""

_DESCRIPTION = """
The e-SNLI dataset extends the Stanford Natural Language Inference Dataset to
include human-annotated natural language explanations of the entailment
relations.
"""
_URL = "https://raw.githubusercontent.com/OanaMariaCamburu/e-SNLI/master/dataset/"


class Esnli(datasets.GeneratorBasedBuilder):
    """e-SNLI: Natural Language Inference with Natural Language Explanations corpus."""

    # Version History
    # 0.0.2 Added explanation_2, explanation_3 fields which exist in the dev/test
    # splits only.
    # 0.0.1 Initial version
    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="plain_text",
            version=datasets.Version("0.0.2"),
            description="Plain text import of e-SNLI",
        )
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "premise": datasets.Value("string"),
                    "hypothesis": datasets.Value("string"),
                    "label": datasets.features.ClassLabel(names=["entailment", "neutral", "contradiction"]),
                    "explanation_1": datasets.Value("string"),
                    "explanation_2": datasets.Value("string"),
                    "explanation_3": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            homepage="https://github.com/OanaMariaCamburu/e-SNLI",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        files = dl_manager.download_and_extract(
            {
                "train": [_URL + "esnli_train_1.csv", _URL + "esnli_train_2.csv"],
                "validation": [_URL + "esnli_dev.csv"],
                "test": [_URL + "esnli_test.csv"],
            }
        )

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"files": files["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"files": files["validation"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"files": files["test"]},
            ),
        ]

    def _generate_examples(self, files):
        """Yields examples."""
        for filepath in files:
            with open(filepath, encoding="utf-8") as f:
                reader = csv.DictReader(f)
                for _, row in enumerate(reader):
                    yield row["pairID"], {
                        "premise": row["Sentence1"],
                        "hypothesis": row["Sentence2"],
                        "label": row["gold_label"],
                        "explanation_1": row["Explanation_1"],
                        "explanation_2": row.get("Explanation_2", ""),
                        "explanation_3": row.get("Explanation_3", ""),
                    }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" Shrinked Turkish NER """


import os

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
"""

_DESCRIPTION = """\
Shrinked version (48 entity type) of the turkish_ner.

Original turkish_ner dataset: Automatically annotated Turkish corpus for named entity recognition and text categorization using large-scale gazetteers. The constructed gazetteers contains approximately 300K entities with thousands of fine-grained entity types under 25 different domains.

Shrinked entity types are: academic, academic_person, aircraft, album_person, anatomy, animal, architect_person, capital, chemical, clothes, country, culture, currency, date, food, genre, government, government_person, language, location, material, measure, medical, military, military_person, nation, newspaper, organization, organization_person, person, production_art_music, production_art_music_person, quantity, religion, science, shape, ship, software, space, space_person, sport, sport_name, sport_person, structure, subject, tech, train, vehicle
"""

_HOMEPAGE = "https://www.kaggle.com/behcetsenturk/shrinked-twnertc-turkish-ner-data-by-kuzgunlar"

_LICENSE = "Attribution 4.0 International (CC BY 4.0)"

_FILENAME = "train.txt"


class TurkishShrinkedNER(datasets.GeneratorBasedBuilder):
    @property
    def manual_download_instructions(self):
        return """\
    You need to go to https://www.kaggle.com/behcetsenturk/shrinked-twnertc-turkish-ner-data-by-kuzgunlar,
    and manually download the turkish_shrinked_ner. Once it is completed,
    a file named archive.zip will be appeared in your Downloads folder
    or whichever folder your browser chooses to save files to. You then have
    to unzip the file and move train.txt under <path/to/folder>.
    The <path/to/folder> can e.g. be "~/manual_data".
    turkish_shrinked_ner can then be loaded using the following command `datasets.load_dataset("turkish_shrinked_ner", data_dir="<path/to/folder>")`.
    """

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "ner_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "O",
                                "B-academic",
                                "I-academic",
                                "B-academic_person",
                                "I-academic_person",
                                "B-aircraft",
                                "I-aircraft",
                                "B-album_person",
                                "I-album_person",
                                "B-anatomy",
                                "I-anatomy",
                                "B-animal",
                                "I-animal",
                                "B-architect_person",
                                "I-architect_person",
                                "B-capital",
                                "I-capital",
                                "B-chemical",
                                "I-chemical",
                                "B-clothes",
                                "I-clothes",
                                "B-country",
                                "I-country",
                                "B-culture",
                                "I-culture",
                                "B-currency",
                                "I-currency",
                                "B-date",
                                "I-date",
                                "B-food",
                                "I-food",
                                "B-genre",
                                "I-genre",
                                "B-government",
                                "I-government",
                                "B-government_person",
                                "I-government_person",
                                "B-language",
                                "I-language",
                                "B-location",
                                "I-location",
                                "B-material",
                                "I-material",
                                "B-measure",
                                "I-measure",
                                "B-medical",
                                "I-medical",
                                "B-military",
                                "I-military",
                                "B-military_person",
                                "I-military_person",
                                "B-nation",
                                "I-nation",
                                "B-newspaper",
                                "I-newspaper",
                                "B-organization",
                                "I-organization",
                                "B-organization_person",
                                "I-organization_person",
                                "B-person",
                                "I-person",
                                "B-production_art_music",
                                "I-production_art_music",
                                "B-production_art_music_person",
                                "I-production_art_music_person",
                                "B-quantity",
                                "I-quantity",
                                "B-religion",
                                "I-religion",
                                "B-science",
                                "I-science",
                                "B-shape",
                                "I-shape",
                                "B-ship",
                                "I-ship",
                                "B-software",
                                "I-software",
                                "B-space",
                                "I-space",
                                "B-space_person",
                                "I-space_person",
                                "B-sport",
                                "I-sport",
                                "B-sport_name",
                                "I-sport_name",
                                "B-sport_person",
                                "I-sport_person",
                                "B-structure",
                                "I-structure",
                                "B-subject",
                                "I-subject",
                                "B-tech",
                                "I-tech",
                                "B-train",
                                "I-train",
                                "B-vehicle",
                                "I-vehicle",
                            ]
                        )
                    ),
                }
            ),
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        path_to_manual_file = os.path.abspath(os.path.expanduser(dl_manager.manual_dir))
        if not os.path.exists(path_to_manual_file):
            raise FileNotFoundError(
                "{path_to_manual_file} does not exist. Make sure you insert a manual dir via `datasets.load_dataset('turkish_shrinked_ner', data_dir=...)` that includes file name {_FILENAME}. Manual download instructions: {self.manual_download_instructions}"
            )
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(path_to_manual_file, "train.txt"),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        logger.info("⏳ Generating examples from = %s", filepath)

        with open(filepath, encoding="utf-8") as f:
            id_ = 0
            tokens = []
            ner_tags = []
            for row in f:
                if row == "":
                    continue
                elif row == "\n":
                    yield id_, {
                        "id": str(id_),
                        "tokens": tokens,
                        "ner_tags": ner_tags,
                    }
                    tokens = []
                    ner_tags = []
                    id_ += 1
                else:
                    token, tag = row.split(" ")
                    tokens.append(token)
                    ner_tags.append(tag)

            if len(tokens) > 0:
                yield id_, {
                    "id": str(id_),
                    "tokens": tokens,
                    "ner_tags": ner_tags,
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""WikiMovies: A Question-Answering dataset that contains raw text alongside a preprocessed knowledge base, in the domain of movies from the Open Movie Database. It is the QA part of the Movie Dialog dataset.
It was built with the following goals in mind: (i) machine learning techniques should have ample training examples for learning; and (ii) one can analyze easily the performance of different representations of knowledge and break down the results by question type. The dataset can be downloaded fromhttp://fb.ai/babi
"""


import datasets


_CITATION = """\
@misc{miller2016keyvalue,
      title={Key-Value Memory Networks for Directly Reading Documents},
      author={Alexander Miller and Adam Fisch and Jesse Dodge and Amir-Hossein Karimi and Antoine Bordes and Jason Weston},
      year={2016},
      eprint={1606.03126},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}"""

_DESCRIPTION = """\
The WikiMovies dataset consists of roughly 100k (templated) questions over 75k entities based on questions with answers in the open movie database (OMDb).
"""

_HOMEPAGE = "https://research.fb.com/downloads/babi/"

_LICENSE = "Creative Commons Public License (CCPL)"

# TODO: Add link to the official dataset URLs here
# The HuggingFace dataset library don't host the datasets but only point to the original files
# This can be an arbitrary nested dict/list of URLs (see below in `_split_generators` method)
_URLs = {"default": "https://thespermwhale.com/jaseweston/babi/movieqa.tar.gz"}


class WikiMovies(datasets.GeneratorBasedBuilder):
    """The WikiMovies dataset consists of roughly 100k (templated) questions over 75k entities based on questions with answers in the open movie database (OMDb)."""

    VERSION = datasets.Version("1.1.0")

    def _info(self):
        features = datasets.Features(
            {
                "question": datasets.Value("string"),
                "answer": datasets.Value("string"),
                # These are the features of your dataset like images, labels ...
            }
        )
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=features,  # Here we define them above because they are different between the two configurations
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO: This method is tasked with downloading/extracting the data and defining the splits depending on the configuration
        # If several configurations are possible (listed in BUILDER_CONFIGS), the configuration selected by the user is in self.config.name

        # dl_manager is a datasets.download.DownloadManager that can be used to download and extract URLs
        # It can accept any type or nested list/dict and will give back the same structure with the url replaced with path to local files.
        # By default the archives will be extracted and a path to a cached folder where they are extracted is returned instead of the archive
        my_urls = _URLs[self.config.name]
        archive = dl_manager.download(my_urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": "/".join(["movieqa", "questions", "wiki_entities", "wiki-entities_qa_train.txt"]),
                    "files": dl_manager.iter_archive(archive),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": "/".join(["movieqa", "questions", "wiki_entities", "wiki-entities_qa_test.txt"]),
                    "files": dl_manager.iter_archive(archive),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": "/".join(["movieqa", "questions", "wiki_entities", "wiki-entities_qa_dev.txt"]),
                    "files": dl_manager.iter_archive(archive),
                },
            ),
        ]

    def _generate_examples(self, filepath, files):
        # It is in charge of opening the given file and yielding (key, example) tuples from the dataset
        # The key is not important, it's more here for legacy reason (legacy from tfds)

        for path, f in files:
            if path == filepath:
                for id_, row in enumerate(f):
                    tmp_data = row.decode("utf-8").split("\t")
                    tmp_question = tmp_data[0][1:]
                    yield id_, {
                        "question": tmp_question,
                        "answer": tmp_data[1],
                    }
                break



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""European Center for Disease Prevention and Control Translation Memory dataset"""


import os
from xml.etree import ElementTree

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@Article{Steinberger2014,
        author={Steinberger, Ralf
                and Ebrahim, Mohamed
                and Poulis, Alexandros
                and Carrasco-Benitez, Manuel
                and Schl{\"u}ter, Patrick
                and Przybyszewski, Marek
                and Gilbro, Signe},
        title={An overview of the European Union's highly multilingual parallel corpora},
        journal={Language Resources and Evaluation},
        year={2014},
        month={Dec},
        day={01},
        volume={48},
        number={4},
        pages={679-707},
        issn={1574-0218},
        doi={10.1007/s10579-014-9277-0},
        url={https://doi.org/10.1007/s10579-014-9277-0}
}
"""

_DESCRIPTION = """\
In October 2012, the European Union (EU) agency 'European Centre for Disease Prevention and Control' (ECDC) released \
a translation memory (TM), i.e. a collection of sentences and their professionally produced translations, in \
twenty-five languages. This resource bears the name EAC Translation Memory, short EAC-TM.
ECDC-TM covers 25 languages: the 23 official languages of the EU plus Norwegian (Norsk) and Icelandic. ECDC-TM was \
created by translating from English into the following 24 languages: Bulgarian, Czech, Danish, Dutch, English, \
Estonian, Gaelige (Irish), German, Greek, Finnish, French, Hungarian, Icelandic, Italian, Latvian, Lithuanian, \
Maltese, Norwegian (NOrsk), Polish, Portuguese, Romanian, Slovak, Slovenian, Spanish and Swedish.
All documents and sentences were thus originally written in English. They were then translated into the other \
languages by professional translators from the Translation Centre CdT in Luxembourg."""

_HOMEPAGE = "https://ec.europa.eu/jrc/en/language-technologies/ecdc-translation-memory"

_LICENSE = "\
Creative Commons Attribution 4.0 International(CC BY 4.0) licence \
Copyright © EU/ECDC, 1995-2020"

_VERSION = "1.0.0"

_DATA_URL = "http://optima.jrc.it/Resources/ECDC-TM/ECDC-TM.zip"

_AVAILABLE_LANGUAGES = (
    "bg",
    "cs",
    "da",
    "de",
    "el",
    "es",
    "en",
    "et",
    "fi",
    "fr",
    "ga",
    "hu",
    "is",
    "it",
    "lt",
    "lv",
    "mt",
    "nl",
    "no",
    "pl",
    "pt",
    "ro",
    "sk",
    "sl",
    "sv",
)


def _find_sentence(translation, language):
    """Util that returns the sentence in the given language from translation, or None if it is not found

    Args:
        translation: `xml.etree.ElementTree.Element`, xml tree element extracted from the translation memory files.
        language: `str`, language of interest e.g. 'en'

    Returns: `str` or `None`, can be `None` if the language of interest is not found in the translation
    """
    # Retrieve the first <tuv> children of translation having xml:lang tag equal to language
    namespaces = {"xml": "http://www.w3.org/XML/1998/namespace"}
    seg_tag = translation.find(path=f".//tuv[@xml:lang='{language.upper()}']/seg", namespaces=namespaces)
    if seg_tag is not None:
        return seg_tag.text
    return None


class EuropaEcdcTMConfig(datasets.BuilderConfig):
    """BuilderConfig for EuropaEcdcTM"""

    def __init__(self, *args, language_pair=(None, None), **kwargs):
        """BuilderConfig for EuropaEcdcTM
        Args:
            language_pair: pair of languages that will be used for translation. Should
                contain 2-letter coded strings. First will be used at source and second
                as target in supervised mode. For example: ("se", "en").
            **kwargs: keyword arguments forwarded to super.
        """
        name = f"{language_pair[0]}2{language_pair[1]}"
        description = f"Translation dataset from {language_pair[0]} to {language_pair[1]}"
        super(EuropaEcdcTMConfig, self).__init__(
            *args,
            name=name,
            description=description,
            **kwargs,
        )
        source, target = language_pair
        assert source != target, "Source and target languages must be different}"
        assert (source in _AVAILABLE_LANGUAGES) and (
            target in _AVAILABLE_LANGUAGES
        ), f"Either source language {source} or target language {target} is not supported. Both must be one of : {_AVAILABLE_LANGUAGES}"
        self.language_pair = language_pair


# TODO: Name of the dataset usually match the script name with CamelCase instead of snake_case
class EuropaEcdcTM(datasets.GeneratorBasedBuilder):
    """European Center for Disease Control and Prevention Translation Memory"""

    BUILDER_CONFIGS = [
        EuropaEcdcTMConfig(language_pair=("en", target), version=_VERSION) for target in ["bg", "fr", "sl"]
    ]
    BUILDER_CONFIG_CLASS = EuropaEcdcTMConfig

    def _info(self):
        source, target = self.config.language_pair
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "translation": datasets.features.Translation(languages=self.config.language_pair),
                }
            ),
            supervised_keys=(source, target),
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        dl_dir = dl_manager.download_and_extract(_DATA_URL)
        filepath = os.path.join(dl_dir, "ECDC-TM", "ECDC.tmx")
        source, target = self.config.language_pair
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": filepath,
                    "source_language": source,
                    "target_language": target,
                },
            ),
        ]

    def _generate_examples(
        self,
        filepath,
        source_language,
        target_language,
    ):
        logger.info(f"⏳ Generating examples from = {filepath}")
        xml_element_tree = ElementTree.parse(filepath)
        xml_body_tag = xml_element_tree.getroot().find("body")
        assert xml_body_tag is not None, f"Invalid data: <body></body> tag not found in {filepath}"

        # Translations are stored in <tu>...</tu> tags
        translation_units = xml_body_tag.iter("tu")

        # _ids may not be contiguous
        for _id, translation in enumerate(translation_units):
            source_sentence = _find_sentence(translation=translation, language=source_language)
            target_sentence = _find_sentence(translation=translation, language=target_language)
            if source_sentence is None or target_sentence is None:
                continue

            yield _id, {
                "translation": {source_language: source_sentence, target_language: target_sentence},
            }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Korean named entity recognition dataset"""


import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@InProceedings{Kim:2016,
  title     = "Korean Named Entity Recognition Dataset",
  authors   = "Jae-Hoon Kim",
  publisher = "GitHub",
  year      = "2016"
}
"""


_DESCRIPTION = """\
Korean named entity recognition dataset
"""

_HOMEPAGE = "https://github.com/kmounlp/NER"

_LICENSE = "NER License, MIT License for non-commercial use"

_URL = "https://raw.githubusercontent.com/kmounlp/NER/master/2016klp/ner."
_URLs = {key: _URL + key for key in ("train", "test", "dev")}


class KorNER(datasets.GeneratorBasedBuilder):
    """Korean Named entity recognition dataset"""

    VERSION = datasets.Version("1.1.0")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "annot_text": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "pos_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "SO",
                                "SS",
                                "VV",
                                "XR",
                                "VCP",
                                "JC",
                                "VCN",
                                "JKB",
                                "MM",
                                "SP",
                                "XSN",
                                "SL",
                                "NNP",
                                "NP",
                                "EP",
                                "JKQ",
                                "IC",
                                "XSA",
                                "EC",
                                "EF",
                                "SE",
                                "XPN",
                                "ETN",
                                "SH",
                                "XSV",
                                "MAG",
                                "SW",
                                "ETM",
                                "JKO",
                                "NNB",
                                "MAJ",
                                "NNG",
                                "JKV",
                                "JKC",
                                "VA",
                                "NR",
                                "JKG",
                                "VX",
                                "SF",
                                "JX",
                                "JKS",
                                "SN",
                            ]
                        )
                    ),
                    "ner_tags": datasets.Sequence(
                        datasets.features.ClassLabel(names=["I", "O", "B_OG", "B_TI", "B_LC", "B_DT", "B_PS"])
                    ),
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        downloaded_files = dl_manager.download_and_extract(_URLs)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": downloaded_files["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": downloaded_files["test"],
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": downloaded_files["dev"],
                    "split": "validation",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        logger.info("⏳ Generating examples from = %s", filepath)
        with open(filepath, encoding="utf-8") as f:
            text = ""
            annot_text = ""
            tokens = []
            pos_tags = []
            ner_tags = []
            for id_, row in enumerate(f):
                row = row.strip()
                if not row:
                    yield id_, {
                        "text": text,
                        "annot_text": annot_text,
                        "tokens": tokens,
                        "pos_tags": pos_tags,
                        "ner_tags": ner_tags,
                    }
                    tokens.clear()
                    pos_tags.clear()
                    ner_tags.clear()
                    continue
                if row[0] == ";":
                    text = row[2:]
                elif row[0] == "$":
                    annot_text = row[1:]
                else:
                    _, token, pos_tag, ner_tag = row.split("\t")
                    tokens.append(token)
                    pos_tags.append(pos_tag)
                    ner_tags.append(ner_tag)



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Rationalized English-French Semantic Divergences (REFreSD) dataset."""


import csv

import datasets


_CITATION = """\
@inproceedings{briakou-carpuat-2020-detecting,
    title = "Detecting Fine-Grained Cross-Lingual Semantic Divergences without Supervision by Learning to Rank",
    author = "Briakou, Eleftheria and Carpuat, Marine",
    booktitle = "Proceedings of the 2020 Conference on Empirical Methods in Natural Language Processing (EMNLP)",
    month = nov,
    year = "2020",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/2020.emnlp-main.121",
    pages = "1563--1580",
}
"""

_DESCRIPTION = """\
The Rationalized English-French Semantic Divergences (REFreSD) dataset consists of 1,039
 English-French sentence-pairs annotated with sentence-level divergence judgments and token-level
 rationales. For any questions, write to ebriakou@cs.umd.edu.
"""

_HOMEPAGE = "https://github.com/Elbria/xling-SemDiv/tree/master/REFreSD"

_LICENSE = "MIT"

_URL = "https://raw.githubusercontent.com/Elbria/xling-SemDiv/master/REFreSD/REFreSD_rationale"


class Refresd(datasets.GeneratorBasedBuilder):
    """The Rationalized English-French Semantic Divergences (REFreSD) dataset."""

    VERSION = datasets.Version("1.1.0")

    def _info(self):
        features = datasets.Features(
            {
                "sentence_pair": datasets.Translation(languages=["en", "fr"]),
                "label": datasets.features.ClassLabel(names=["divergent", "equivalent"]),
                "all_labels": datasets.features.ClassLabel(
                    names=["unrelated", "some_meaning_difference", "no_meaning_difference"]
                ),
                "rationale_en": datasets.features.Sequence(datasets.Value("int32")),
                "rationale_fr": datasets.features.Sequence(datasets.Value("int32")),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        my_urls = _URL
        data_file_path = dl_manager.download_and_extract(my_urls)
        return [datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": data_file_path})]

    def _generate_examples(self, filepath):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as f:
            reader = csv.DictReader(f, delimiter="\t", quoting=csv.QUOTE_NONE)
            for idx, row in enumerate(reader):
                yield idx, {
                    "sentence_pair": {"fr": row["#french_sentence"], "en": row["#english_sentence"]},
                    "label": row["#binary_label"],
                    "all_labels": row["#3_labels"],
                    "rationale_en": [int(v) for v in row["#english_rational"].split(" ")],
                    "rationale_fr": [int(v) for v in row["#french_rationale"].split(" ")],
                }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Tiny Shakespeare dataset."""


import os

import datasets


_CITATION = """\
@misc{
  author={Karpathy, Andrej},
  title={char-rnn},
  year={2015},
  howpublished={\\url{https://github.com/karpathy/char-rnn}}
}"""

_DESCRIPTION = """\
40,000 lines of Shakespeare from a variety of Shakespeare's plays. \
Featured in Andrej Karpathy's blog post 'The Unreasonable Effectiveness of \
Recurrent Neural Networks': \
http://karpathy.github.io/2015/05/21/rnn-effectiveness/.

To use for e.g. character modelling:

```
d = datasets.load_dataset(name='tiny_shakespeare')['train']
d = d.map(lambda x: datasets.Value('strings').unicode_split(x['text'], 'UTF-8'))
# train split includes vocabulary for other splits
vocabulary = sorted(set(next(iter(d)).numpy()))
d = d.map(lambda x: {'cur_char': x[:-1], 'next_char': x[1:]})
d = d.unbatch()
seq_len = 100
batch_size = 2
d = d.batch(seq_len)
d = d.batch(batch_size)
```
"""


class TinyShakespeare(datasets.GeneratorBasedBuilder):
    """Tiny Shakespeare dataset builder."""

    VERSION = datasets.Version("1.0.0")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features({"text": datasets.Value("string")}),
            supervised_keys=None,
            homepage="https://github.com/karpathy/char-rnn/blob/master/data/tinyshakespeare/input.txt",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        download_path = dl_manager.download_and_extract(
            "https://raw.githubusercontent.com/karpathy/char-rnn/master/data/tinyshakespeare/input.txt"
        )
        if os.path.isdir(download_path):
            # During testing the download manager mock gives us a directory
            txt_path = os.path.join(download_path, "input.txt")
        else:
            txt_path = download_path
        with open(txt_path, "r", encoding="utf-8") as f:
            text = f.read()

        # 90/5/5 split
        i = int(len(text) * 0.9)
        train_text, text = text[:i], text[i:]
        i = int(len(text) * 0.5)
        validation_text, text = text[:i], text[i:]
        test_text = text

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"split_key": "train", "split_text": train_text},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"split_key": "validation", "split_text": validation_text},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"split_key": "test", "split_text": test_text},
            ),
        ]

    def _generate_examples(self, split_key, split_text):
        """Yields examples."""
        data_key = split_key  # Should uniquely identify the thing yielded
        feature_dict = {"text": split_text}
        yield data_key, feature_dict



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
import datasets


_DESCRIPTION = """\
Parallel Igbo-English Dataset
"""
_HOMEPAGE_URL = "https://github.com/IgnatiusEzeani/IGBONLP/tree/master/ig_en_mt"
_CITATION = """\
@misc{ezeani2020igboenglish,
    title={Igbo-English Machine Translation: An Evaluation Benchmark},
    author={Ignatius Ezeani and Paul Rayson and Ikechukwu Onyenwe and Chinedu Uchechukwu and Mark Hepple},
    year={2020},
    eprint={2004.00648},
    archivePrefix={arXiv},
    primaryClass={cs.CL},
    url={https://arxiv.org/abs/2004.00648}
}
"""

_VERSION = "1.0.0"
_TRAIN_EN = "https://raw.githubusercontent.com/IgnatiusEzeani/IGBONLP/master/ig_en_mt/benchmark_dataset/train.en"
_VALID_EN = "https://raw.githubusercontent.com/IgnatiusEzeani/IGBONLP/master/ig_en_mt/benchmark_dataset/val.en"
_TEST_EN = "https://raw.githubusercontent.com/IgnatiusEzeani/IGBONLP/master/ig_en_mt/benchmark_dataset/test.en"
_TRAIN_IG = "https://raw.githubusercontent.com/IgnatiusEzeani/IGBONLP/master/ig_en_mt/benchmark_dataset/train.ig"
_VALID_IG = "https://raw.githubusercontent.com/IgnatiusEzeani/IGBONLP/master/ig_en_mt/benchmark_dataset/val.ig"
_TEST_IG = "https://raw.githubusercontent.com/IgnatiusEzeani/IGBONLP/master/ig_en_mt/benchmark_dataset/test.ig"

_LANGUAGE_PAIRS = [
    ("ig", "en"),
]


class IgboEnglishMachineTranslationConfig(datasets.BuilderConfig):
    def __init__(self, *args, lang1=None, lang2=None, **kwargs):
        super().__init__(
            *args,
            name=f"{lang1}-{lang2}",
            **kwargs,
        )
        self.lang1 = lang1
        self.lang2 = lang2


class IgboEnglishMachineTranslation(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        IgboEnglishMachineTranslationConfig(
            lang1=lang1,
            lang2=lang2,
            description=f"Translating {lang1} to {lang2} or vice versa",
            version=datasets.Version(_VERSION),
        )
        for lang1, lang2 in _LANGUAGE_PAIRS
    ]
    BUILDER_CONFIG_CLASS = IgboEnglishMachineTranslationConfig

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "translation": datasets.Translation(languages=(self.config.lang1, self.config.lang2)),
                },
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        train_en = dl_manager.download_and_extract(_TRAIN_EN)
        train_ig = dl_manager.download_and_extract(_TRAIN_IG)

        valid_en = dl_manager.download_and_extract(_VALID_EN)
        valid_ig = dl_manager.download_and_extract(_VALID_IG)

        test_en = dl_manager.download_and_extract(_TEST_EN)
        test_ig = dl_manager.download_and_extract(_TEST_IG)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"ig_datapath": train_ig, "en_datapath": train_en},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"ig_datapath": valid_ig, "en_datapath": valid_en},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"ig_datapath": test_ig, "en_datapath": test_en},
            ),
        ]

    def _generate_examples(self, ig_datapath, en_datapath):
        with open(ig_datapath, encoding="utf-8") as f1, open(en_datapath, encoding="utf-8") as f2:
            for sentence_counter, (x, y) in enumerate(zip(f1, f2)):
                x = x.strip()
                y = y.strip()
                result = (
                    sentence_counter,
                    {
                        "id": str(sentence_counter),
                        "translation": {"ig": x, "en": y},
                    },
                )
                yield result



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""CLICK-ID: A Novel Dataset for Indonesian Clickbait Headlines"""


import csv
import glob
import os

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@inproceedings{id_clickbait,
  author    = {Andika William, Yunita Sari},
  title     = {CLICK-ID: A Novel Dataset for Indonesian Clickbait Headlines},
  year      = {2020},
  url       = {http://dx.doi.org/10.17632/k42j7x2kpn.1},
}
"""

_DESCRIPTION = """\
The CLICK-ID dataset is a collection of Indonesian news headlines that was collected from 12 local online news
publishers; detikNews, Fimela, Kapanlagi, Kompas, Liputan6, Okezone, Posmetro-Medan, Republika, Sindonews, Tempo,
Tribunnews, and Wowkeren. This dataset is comprised of mainly two parts; (i) 46,119 raw article data, and (ii)
15,000 clickbait annotated sample headlines. Annotation was conducted with 3 annotator examining each headline.
Judgment were based only on the headline. The majority then is considered as the ground truth. In the annotated
sample, our annotation shows 6,290 clickbait and 8,710 non-clickbait.
"""

_HOMEPAGE = "https://github.com/feryandi/Dataset-Artikel"

_LICENSE = "Creative Commons Attribution 4.0 International license"

_URLs = ["https://md-datasets-cache-zipfiles-prod.s3.eu-west-1.amazonaws.com/k42j7x2kpn-1.zip"]


class IdClickbaitConfig(datasets.BuilderConfig):
    """BuilderConfig for IdClickbait"""

    def __init__(self, label_classes=None, path=None, **kwargs):
        """BuilderConfig for IdClickbait.
        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(IdClickbaitConfig, self).__init__(**kwargs)
        self.label_classes = label_classes
        self.path = path


class IdClickbait(datasets.GeneratorBasedBuilder):
    VERSION = datasets.Version("1.0.0")

    BUILDER_CONFIGS = [
        IdClickbaitConfig(
            name="annotated",
            version=VERSION,
            description="Annotated clickbait dataset",
            label_classes=["non-clickbait", "clickbait"],
            path="annotated/csv",
        ),
        IdClickbaitConfig(name="raw", version=VERSION, description="Raw dataset", path="raw/csv"),
    ]

    BUILDER_CONFIG_CLASS = IdClickbaitConfig

    def _info(self):
        if self.config.name == "annotated":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "label": datasets.features.ClassLabel(names=self.config.label_classes),
                }
            )
        else:
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "source": datasets.Value("string"),
                    "date": datasets.Value("string"),
                    "category": datasets.Value("string"),
                    "sub-category": datasets.Value("string"),
                    "content": datasets.Value("string"),
                    "url": datasets.Value("string"),
                }
            )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        my_urls = _URLs[0]
        data_dir = dl_manager.download_and_extract(my_urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "article_dir": os.path.join(data_dir, self.config.path),
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, article_dir, split):
        logger.info("⏳ Generating %s examples from = %s", split, article_dir)
        id = 0
        for path in sorted(glob.glob(os.path.join(article_dir, "**/*.csv"), recursive=True)):
            with open(path, encoding="utf-8-sig", newline="") as f:
                reader = csv.DictReader(f)
                for row in reader:
                    if self.config.name == "annotated":
                        yield id, {
                            "id": str(id),
                            "title": row["title"],
                            "label": row["label"],
                        }
                    else:
                        yield id, {
                            "id": str(id),
                            "title": row["title"],
                            "source": row["source"],
                            "date": row["date"],
                            "category": row["category"],
                            "sub-category": row["sub-category"],
                            "content": row["content"],
                            "url": row["url"],
                        }
                    id += 1



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""A Dataset of Peer Reviews (PeerRead): Collection, Insights and NLP Applications"""


import glob
import json
import os

import datasets


_CITATION = """\
@inproceedings{kang18naacl,
  title = {A Dataset of Peer Reviews (PeerRead): Collection, Insights and NLP Applications},
  author = {Dongyeop Kang and Waleed Ammar and Bhavana Dalvi and Madeleine van Zuylen and Sebastian Kohlmeier and Eduard Hovy and Roy Schwartz},
  booktitle = {Meeting of the North American Chapter of the Association for Computational Linguistics (NAACL)},
  address = {New Orleans, USA},
  month = {June},
  url = {https://arxiv.org/abs/1804.09635},
  year = {2018}
}
"""

_DESCRIPTION = """\
PearRead is a dataset of scientific peer reviews available to help researchers study this important artifact. The dataset consists of over 14K paper drafts and the corresponding accept/reject decisions in top-tier venues including ACL, NIPS and ICLR, as well as over 10K textual peer reviews written by experts for a subset of the papers.
"""

_HOMEPAGE = "https://github.com/allenai/PeerRead"

_LICENSE = "Creative Commons Public License"

_URLs = {
    "dataset_repo": "https://github.com/allenai/PeerRead/archive/master.zip",
}


class PeerRead(datasets.GeneratorBasedBuilder):
    """A Dataset of Peer Reviews (PeerRead): Collection, Insights and NLP Applications"""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="parsed_pdfs",
            version=VERSION,
            description="Research paper drafts",
        ),
        datasets.BuilderConfig(
            name="reviews",
            version=VERSION,
            description="Accept/reject decisions in top-tier venues including ACL, NIPS and ICLR",
        ),
    ]

    @staticmethod
    def _get_paths(data_dir, domain):
        paths = {"train": [], "test": [], "dev": []}
        conference_paths = glob.glob(os.path.join(data_dir, "PeerRead-master/data/*"))
        for conference_path in conference_paths:
            for dtype in ["test", "train", "dev"]:
                file_paths = glob.glob(os.path.join(conference_path, dtype, domain, "*.json"))
                for file_path in file_paths:
                    paths[dtype].append(file_path)
        return paths

    @staticmethod
    def _parse_histories(histories):
        if histories is None:
            return [[]]
        if isinstance(histories, str):
            return [[histories]]
        return histories

    @staticmethod
    def _parse_reviews(data):
        reviews = []
        for review in data.get("metadata", {}).get("reviews", []):
            if isinstance(review, dict):
                reviews.append(
                    {
                        "date": str(review.get("date", "")),
                        "title": str(review.get("title", "")),
                        "other_keys": str(review.get("other_keys", "")),
                        "originality": str(review.get("originality", "")),
                        "comments": str(review.get("comments", "")),
                        "is_meta_review": str(review.get("is_meta_review", "")),
                        "is_annotated": str(review.get("is_annotated", "")),
                        "recommendation": str(review.get("recommendation", "")),
                        "replicability": str(review.get("replicability", "")),
                        "presentation_format": str(review.get("presentation_format", "")),
                        "clarity": str(review.get("clarity", "")),
                        "meaningful_comparison": str(review.get("meaningful_comparison", "")),
                        "substance": str(review.get("substance", "")),
                        "reviewer_confidence": str(review.get("reviewer_confidence", "")),
                        "soundness_correctness": str(review.get("soundness_correctness", "")),
                        "appropriateness": str(review.get("appropriateness", "")),
                        "impact": str(review.get("impact")),
                    }
                )
        return reviews

    @staticmethod
    def _decode(text):
        return str(text).encode("utf-8", "replace").decode("utf-8")

    def _info(self):
        if (
            self.config.name == "parsed_pdfs"
        ):  # This is the name of the configuration selected in BUILDER_CONFIGS above
            features = datasets.Features(
                {
                    "name": datasets.Value("string"),
                    "metadata": {
                        "source": datasets.Value("string"),
                        "title": datasets.Value("string"),
                        "authors": datasets.features.Sequence(datasets.Value("string")),
                        "emails": datasets.features.Sequence(datasets.Value("string")),
                        "sections": datasets.features.Sequence(
                            {
                                "heading": datasets.Value("string"),
                                "text": datasets.Value("string"),
                            }
                        ),
                        "references": datasets.features.Sequence(
                            {
                                "title": datasets.Value("string"),
                                "author": datasets.features.Sequence(datasets.Value("string")),
                                "venue": datasets.Value("string"),
                                "citeRegEx": datasets.Value("string"),
                                "shortCiteRegEx": datasets.Value("string"),
                                "year": datasets.Value("int32"),
                            }
                        ),
                        "referenceMentions": datasets.features.Sequence(
                            {
                                "referenceID": datasets.Value("int32"),
                                "context": datasets.Value("string"),
                                "startOffset": datasets.Value("int32"),
                                "endOffset": datasets.Value("int32"),
                            }
                        ),
                        "year": datasets.Value("int32"),
                        "abstractText": datasets.Value("string"),
                        "creator": datasets.Value("string"),
                    },
                }
            )
        else:
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "conference": datasets.Value("string"),
                    "comments": datasets.Value("string"),
                    "subjects": datasets.Value("string"),
                    "version": datasets.Value("string"),
                    "date_of_submission": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "authors": datasets.features.Sequence(datasets.Value("string")),
                    "accepted": datasets.Value("bool"),
                    "abstract": datasets.Value("string"),
                    "histories": datasets.features.Sequence(datasets.features.Sequence(datasets.Value("string"))),
                    "reviews": datasets.features.Sequence(
                        {
                            "date": datasets.Value("string"),
                            "title": datasets.Value("string"),
                            "other_keys": datasets.Value("string"),
                            "originality": datasets.Value("string"),
                            "comments": datasets.Value("string"),
                            "is_meta_review": datasets.Value("bool"),
                            "is_annotated": datasets.Value("bool"),
                            "recommendation": datasets.Value("string"),
                            "replicability": datasets.Value("string"),
                            "presentation_format": datasets.Value("string"),
                            "clarity": datasets.Value("string"),
                            "meaningful_comparison": datasets.Value("string"),
                            "substance": datasets.Value("string"),
                            "reviewer_confidence": datasets.Value("string"),
                            "soundness_correctness": datasets.Value("string"),
                            "appropriateness": datasets.Value("string"),
                            "impact": datasets.Value("string"),
                        }
                    ),
                }
            )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        url = _URLs["dataset_repo"]
        data_dir = dl_manager.download_and_extract(url)
        paths = self._get_paths(
            data_dir=data_dir,
            domain=self.config.name,
        )

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepaths": paths["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepaths": paths["test"], "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepaths": paths["dev"],
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepaths, split):
        """Yields examples."""
        for id_, filepath in enumerate(sorted(filepaths)):
            with open(filepath, encoding="utf-8", errors="replace") as f:
                data = json.load(f)
                if self.config.name == "parsed_pdfs":
                    metadata = data.get(
                        "metadata",
                        {
                            "source": "",
                            "authors": [],
                            "title": [],
                            "sections": [],
                            "references": [],
                            "referenceMentions": [],
                            "year": "",
                            "abstractText": "",
                            "creator": "",
                        },
                    )
                    metadata["sections"] = [] if metadata["sections"] is None else metadata["sections"]
                    metadata["sections"] = [
                        {
                            "heading": self._decode(section.get("heading", "")),
                            "text": self._decode(section.get("text", "")),
                        }
                        for section in metadata["sections"]
                    ]
                    metadata["references"] = [] if metadata["references"] is None else metadata["references"]
                    metadata["references"] = [
                        {
                            "title": reference.get("title", ""),
                            "author": reference.get("author", []),
                            "venue": reference.get("venue", ""),
                            "citeRegEx": reference.get("citeRegEx", ""),
                            "shortCiteRegEx": reference.get("shortCiteRegEx", ""),
                            "year": reference.get("year", ""),
                        }
                        for reference in metadata["references"]
                    ]
                    metadata["referenceMentions"] = (
                        [] if metadata["referenceMentions"] is None else metadata["referenceMentions"]
                    )
                    metadata["referenceMentions"] = [
                        {
                            "referenceID": self._decode(reference_mention.get("referenceID", "")),
                            "context": self._decode(reference_mention.get("context", "")),
                            "startOffset": self._decode(reference_mention.get("startOffset", "")),
                            "endOffset": self._decode(reference_mention.get("endOffset", "")),
                        }
                        for reference_mention in metadata["referenceMentions"]
                    ]

                    yield id_, {
                        "name": data.get("name", ""),
                        "metadata": metadata,
                    }
                elif self.config.name == "reviews":
                    yield id_, {
                        "id": str(data.get("id", "")),
                        "conference": str(data.get("conference", "")),
                        "comments": str(data.get("comments", "")),
                        "subjects": str(data.get("subjects", "")),
                        "version": str(data.get("version", "")),
                        "date_of_submission": str(data.get("date_of_submission", "")),
                        "title": str(data.get("title", "")),
                        "authors": data.get("authors", [])
                        if isinstance(data.get("authors"), list)
                        else ([data.get("authors")] if data.get("authors") else []),
                        "accepted": str(data.get("accepted", "")),
                        "abstract": str(data.get("abstract", "")),
                        "histories": self._parse_histories(data.get("histories", [])),
                        "reviews": self._parse_reviews(data),
                    }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Taskmaster: A dataset for goal oriented conversations."""


import json

import datasets


_CITATION = """\
@inproceedings{48484,
title	= {Taskmaster-1: Toward a Realistic and Diverse Dialog Dataset},
author	= {Bill Byrne and Karthik Krishnamoorthi and Chinnadhurai Sankar and Arvind Neelakantan and Daniel Duckworth and Semih Yavuz and Ben Goodrich and Amit Dubey and Kyu-Young Kim and Andy Cedilnik},
year	= {2019}
}
"""

_DESCRIPTION = """\
Taskmaster is dataset for goal oriented conversations. The Taskmaster-2 dataset consists of 17,289 dialogs \
in the seven domains which include restaurants, food ordering, movies, hotels, flights, music and sports. \
Unlike Taskmaster-1, which includes both written "self-dialogs" and spoken two-person dialogs, \
Taskmaster-2 consists entirely of spoken two-person dialogs. In addition, while Taskmaster-1 is \
almost exclusively task-based, Taskmaster-2 contains a good number of search- and recommendation-oriented dialogs. \
All dialogs in this release were created using a Wizard of Oz (WOz) methodology in which crowdsourced \
workers played the role of a 'user' and trained call center operators played the role of the 'assistant'. \
In this way, users were led to believe they were interacting with an automated system that “spoke” \
using text-to-speech (TTS) even though it was in fact a human behind the scenes. \
As a result, users could express themselves however they chose in the context of an automated interface.
"""

_HOMEPAGE = "https://github.com/google-research-datasets/Taskmaster/tree/master/TM-2-2020"

_BASE_URL = "https://raw.githubusercontent.com/google-research-datasets/Taskmaster/master/TM-2-2020/data"


class Taskmaster2(datasets.GeneratorBasedBuilder):
    """Taskmaster: A dataset for goal oriented conversations."""

    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="flights", version=datasets.Version("1.0.0"), description="Taskmaster-2 flights domain."
        ),
        datasets.BuilderConfig(
            name="food-ordering", version=datasets.Version("1.0.0"), description="Taskmaster-2 food-ordering domain"
        ),
        datasets.BuilderConfig(
            name="hotels", version=datasets.Version("1.0.0"), description="Taskmaster-2 hotel domain"
        ),
        datasets.BuilderConfig(
            name="movies", version=datasets.Version("1.0.0"), description="Taskmaster-2 movies domain"
        ),
        datasets.BuilderConfig(
            name="music", version=datasets.Version("1.0.0"), description="Taskmaster-2 music domain"
        ),
        datasets.BuilderConfig(
            name="restaurant-search",
            version=datasets.Version("1.0.0"),
            description="Taskmaster-2 restaurant-search domain",
        ),
        datasets.BuilderConfig(
            name="sports", version=datasets.Version("1.0.0"), description="Taskmaster-2 sports domain"
        ),
    ]

    def _info(self):
        features = {
            "conversation_id": datasets.Value("string"),
            "instruction_id": datasets.Value("string"),
            "utterances": [
                {
                    "index": datasets.Value("int32"),
                    "speaker": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "segments": [
                        {
                            "start_index": datasets.Value("int32"),
                            "end_index": datasets.Value("int32"),
                            "text": datasets.Value("string"),
                            "annotations": [{"name": datasets.Value("string")}],
                        }
                    ],
                }
            ],
        }
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(features),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        url = f"{_BASE_URL}/{self.config.name}.json"
        dialogs_file = dl_manager.download(url)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": dialogs_file},
            ),
        ]

    def _generate_examples(self, filepath):
        with open(filepath, encoding="utf-8") as f:
            dialogs = json.load(f)
            for dialog in dialogs:
                utterances = dialog["utterances"]
                for utterance in utterances:
                    if "segments" not in utterance:
                        utterance["segments"] = []
                yield dialog["conversation_id"], dialog



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""MedHop: Reading Comprehension with Multiple Hops"""


import json
import os

import datasets


_CITATION = """\
@misc{welbl2018constructing,
      title={Constructing Datasets for Multi-hop Reading Comprehension Across Documents},
      author={Johannes Welbl and Pontus Stenetorp and Sebastian Riedel},
      year={2018},
      eprint={1710.06481},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

_DESCRIPTION = """\
MedHop is based on research paper abstracts from PubMed, and the queries are about interactions between pairs of drugs. \
The correct answer has to be inferred by combining information from a chain of reactions of drugs and proteins.
"""

_URL = "https://drive.google.com/uc?export=download&id=1ytVZ4AhubFDOEL7o7XrIRIyhU8g9wvKA"


class MedHopConfig(datasets.BuilderConfig):
    """BuilderConfig for MedHop."""

    def __init__(self, masked=False, **kwargs):
        """BuilderConfig for MedHop.

        Args:
          masked: `bool`, original or maksed data.
          **kwargs: keyword arguments forwarded to super.
        """
        super(MedHopConfig, self).__init__(**kwargs)
        self.masked = masked


class MedHop(datasets.GeneratorBasedBuilder):
    """MedHop: Reading Comprehension with Multiple Hops"""

    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIGS = [
        MedHopConfig(
            name="original",
            version=datasets.Version("1.0.0"),
            description="The un-maksed MedHop dataset",
            masked=False,
        ),
        MedHopConfig(
            name="masked", version=datasets.Version("1.0.0"), description="Masked MedHop dataset", masked=True
        ),
    ]
    BUILDER_CONFIG_CLASS = MedHopConfig
    DEFAULT_CONFIG_NAME = "original"

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "answer": datasets.Value("string"),
                    "candidates": datasets.Sequence(datasets.Value("string")),
                    "supports": datasets.Sequence(datasets.Value("string")),
                }
            ),
            supervised_keys=None,
            homepage="http://qangaroo.cs.ucl.ac.uk/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        extracted_path = dl_manager.download_and_extract(_URL)

        medhop_path = os.path.join(extracted_path, "qangaroo_v1.1", "medhop")
        train_file = "train.json" if self.config.name == "original" else "train.masked.json"
        dev_file = "dev.json" if self.config.name == "original" else "dev.masked.json"

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": os.path.join(medhop_path, train_file)},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": os.path.join(medhop_path, dev_file)},
            ),
        ]

    def _generate_examples(self, filepath):
        with open(filepath, encoding="utf-8") as f:
            examples = json.load(f)
            for i, example in enumerate(examples):
                example["question"] = example.pop("query")
                yield example["id"], example



"""Allocine Dataset: A Large-Scale French Movie Reviews Dataset."""


import json
import os

import datasets
from datasets.tasks import TextClassification


_CITATION = """\
@misc{blard2019allocine,
  author = {Blard, Theophile},
  title = {french-sentiment-analysis-with-bert},
  year = {2020},
  publisher = {GitHub},
  journal = {GitHub repository},
  howpublished={\\url{https://github.com/TheophileBlard/french-sentiment-analysis-with-bert}},
}
"""

_DESCRIPTION = """\
 Allocine Dataset: A Large-Scale French Movie Reviews Dataset.
 This is a dataset for binary sentiment classification, made of user reviews scraped from Allocine.fr.
 It contains 100k positive and 100k negative reviews divided into 3 balanced splits: train (160k reviews), val (20k) and test (20k).
"""


class AllocineConfig(datasets.BuilderConfig):
    """BuilderConfig for Allocine."""

    def __init__(self, **kwargs):
        """BuilderConfig for Allocine.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(AllocineConfig, self).__init__(**kwargs)


class AllocineDataset(datasets.GeneratorBasedBuilder):
    """Allocine Dataset: A Large-Scale French Movie Reviews Dataset."""

    _DOWNLOAD_URL = "https://github.com/TheophileBlard/french-sentiment-analysis-with-bert/raw/master/allocine_dataset/data.tar.bz2"
    _TRAIN_FILE = "train.jsonl"
    _VAL_FILE = "val.jsonl"
    _TEST_FILE = "test.jsonl"

    BUILDER_CONFIGS = [
        AllocineConfig(
            name="allocine",
            version=datasets.Version("1.0.0"),
            description="Allocine Dataset: A Large-Scale French Movie Reviews Dataset",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "review": datasets.Value("string"),
                    "label": datasets.features.ClassLabel(names=["neg", "pos"]),
                }
            ),
            supervised_keys=None,
            homepage="https://github.com/TheophileBlard/french-sentiment-analysis-with-bert",
            citation=_CITATION,
            task_templates=[TextClassification(text_column="review", label_column="label")],
        )

    def _split_generators(self, dl_manager):
        arch_path = dl_manager.download_and_extract(self._DOWNLOAD_URL)
        data_dir = os.path.join(arch_path, "data")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN, gen_kwargs={"filepath": os.path.join(data_dir, self._TRAIN_FILE)}
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION, gen_kwargs={"filepath": os.path.join(data_dir, self._VAL_FILE)}
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST, gen_kwargs={"filepath": os.path.join(data_dir, self._TEST_FILE)}
            ),
        ]

    def _generate_examples(self, filepath):
        """Generate Allocine examples."""
        with open(filepath, encoding="utf-8") as f:
            for id_, row in enumerate(f):
                data = json.loads(row)
                review = data["review"]
                label = "neg" if data["polarity"] == 0 else "pos"
                yield id_, {"review": review, "label": label}



"""`generated_reviews_enth`: Generated product reviews dataset for machine translation quality prediction, part of [scb-mt-en-th-2020](https://arxiv.org/pdf/2007.03541.pdf)"""


import json
import os

import datasets


_CITATION = """\
@article{lowphansirikul2020scb,
  title={scb-mt-en-th-2020: A Large English-Thai Parallel Corpus},
  author={Lowphansirikul, Lalita and Polpanumas, Charin and Rutherford, Attapol T and Nutanong, Sarana},
  journal={arXiv preprint arXiv:2007.03541},
  year={2020}
}
"""

_DESCRIPTION = """\
 `generated_reviews_enth`
 Generated product reviews dataset for machine translation quality prediction, part of [scb-mt-en-th-2020](https://arxiv.org/pdf/2007.03541.pdf)
 `generated_reviews_enth` is created as part of [scb-mt-en-th-2020](https://arxiv.org/pdf/2007.03541.pdf) for machine translation task.
 This dataset (referred to as `generated_reviews_yn` in [scb-mt-en-th-2020](https://arxiv.org/pdf/2007.03541.pdf)) are English product reviews
 generated by [CTRL](https://arxiv.org/abs/1909.05858), translated by Google Translate API and annotated as accepted or rejected (`correct`)
 based on fluency and adequacy of the translation by human annotators.
 This allows it to be used for English-to-Thai translation quality esitmation (binary label), machine translation, and sentiment analysis.
"""


class GeneratedReviewsEnthConfig(datasets.BuilderConfig):
    """BuilderConfig for GeneratedReviewsEnth."""

    def __init__(self, **kwargs):
        """BuilderConfig for GeneratedReviewsEnth.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(GeneratedReviewsEnthConfig, self).__init__(**kwargs)
        self.language_pair = ("en", "th")


class GeneratedReviewsEnth(datasets.GeneratorBasedBuilder):
    """`generated_reviews_enth`: Generated product reviews dataset for machine translation quality prediction, part of [scb-mt-en-th-2020](https://arxiv.org/pdf/2007.03541.pdf)"""

    _DOWNLOAD_URL = "https://github.com/vistec-AI/generated_reviews_enth/raw/main/data.zip"
    _TRAIN_FILE = "train.jsonl"
    _VAL_FILE = "valid.jsonl"
    _TEST_FILE = "test.jsonl"
    BUILDER_CONFIG_CLASS = GeneratedReviewsEnthConfig
    BUILDER_CONFIGS = [
        GeneratedReviewsEnthConfig(
            name="generated_reviews_enth",
            version=datasets.Version("1.0.0"),
            description="`generated_reviews_enth`: Generated product reviews dataset for machine translation quality prediction, part of [scb-mt-en-th-2020](https://arxiv.org/pdf/2007.03541.pdf)",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "translation": datasets.features.Translation(languages=self.config.language_pair),
                    "review_star": datasets.Value("int32"),
                    "correct": datasets.features.ClassLabel(names=["neg", "pos"]),
                }
            ),
            supervised_keys=None,
            homepage="https://github.com/vistec-AI/generated_reviews_enth",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        arch_path = dl_manager.download_and_extract(self._DOWNLOAD_URL)
        data_dir = os.path.join(arch_path, "data")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN, gen_kwargs={"filepath": os.path.join(data_dir, self._TRAIN_FILE)}
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION, gen_kwargs={"filepath": os.path.join(data_dir, self._VAL_FILE)}
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST, gen_kwargs={"filepath": os.path.join(data_dir, self._TEST_FILE)}
            ),
        ]

    def _generate_examples(self, filepath):
        """Generate generated_reviews_enth examples."""
        source, target = self.config.language_pair
        with open(filepath, encoding="utf-8") as f:
            for id_, row in enumerate(f):
                data = json.loads(row)
                yield id_, {
                    "translation": {source: data["en_segment"], target: data["th_segment"]},
                    "review_star": data["review_star"],
                    "correct": data["correct"],
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Google Sentence Compression dataset"""


import gzip
import json

import datasets


_CITATION = """\
@inproceedings{filippova-altun-2013-overcoming,
    title = "Overcoming the Lack of Parallel Data in Sentence Compression",
    author = "Filippova, Katja  and
      Altun, Yasemin",
    booktitle = "Proceedings of the 2013 Conference on Empirical Methods in Natural Language Processing",
    month = oct,
    year = "2013",
    address = "Seattle, Washington, USA",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/D13-1155",
    pages = "1481--1491",
}
"""

_DESCRIPTION = """\
Large corpus of uncompressed and compressed sentences from news articles.
"""

_HOMEPAGE = "https://github.com/google-research-datasets/sentence-compression"


_URLs = {
    datasets.Split.VALIDATION: [
        "https://github.com/google-research-datasets/sentence-compression/raw/master/data/comp-data.eval.json.gz"
    ],
    datasets.Split.TRAIN: [
        f"https://github.com/google-research-datasets/sentence-compression/raw/master/data/sent-comp.train{str(i).zfill(2)}.json.gz"
        for i in range(1, 11)
    ],
}


class SentComp(datasets.GeneratorBasedBuilder):
    """Google Setence Compression dataset"""

    def _info(self):
        node_features = {
            "form": datasets.Value("string"),
            "type": datasets.Value("string"),
            "mid": datasets.Value("string"),
            "word": datasets.features.Sequence(
                {
                    "id": datasets.Value("int32"),
                    "form": datasets.Value("string"),
                    "stem": datasets.Value("string"),
                    "tag": datasets.Value("string"),
                }
            ),
            "gender": datasets.Value("int32"),
            "head_word_index": datasets.Value("int32"),
        }
        compression_edge_features = {
            "parent_id": datasets.Value("int32"),
            "child_id": datasets.Value("int32"),
        }
        edge_features = {**compression_edge_features, "label": datasets.Value("string")}
        entity_features = {
            "start": datasets.Value("int32"),
            "end": datasets.Value("int32"),
            "head": datasets.Value("int32"),
            "name": datasets.Value("string"),
            "type": datasets.Value("string"),
            "mid": datasets.Value("string"),
            "is_proper_name_entity": datasets.Value("bool"),
            "gender": datasets.Value("int32"),
        }
        tree_features = {
            "id": datasets.Value("string"),
            "sentence": datasets.Value("string"),
            "node": datasets.features.Sequence(node_features),
            "edge": datasets.features.Sequence(edge_features),
            "entity_mention": datasets.features.Sequence(entity_features),
        }
        compression_features = {
            "text": datasets.Value("string"),
            "edge": datasets.features.Sequence(compression_edge_features),
        }

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "graph": tree_features,
                    "compression": compression_features,
                    "headline": datasets.Value("string"),
                    "compression_ratio": datasets.Value("float"),
                    "doc_id": datasets.Value("string"),
                    "source_tree": tree_features,
                    "compression_untransformed": compression_features,
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        return [
            datasets.SplitGenerator(
                name=split,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepaths": dl_manager.download(_URLs[split])},
            )
            for split in _URLs
        ]

    def _generate_examples(self, filepaths):
        """Yields examples."""
        id_ = -1
        for ix, filepath in enumerate(filepaths):
            with gzip.open(filepath, mode="rt", encoding="utf-8") as f:
                all_text = f.read()

            # in the data file, it's in the form of JSON objects, separated with '\n\n' characters
            # we'll format the file to be able to read with json package
            all_text = "[" + all_text + "]"
            all_text = all_text.replace("}\n\n{", "},\n{")

            samples = json.loads(all_text)
            for sample in samples:
                # add some default values
                for node in sample["graph"]["node"] + sample["source_tree"]["node"]:
                    if "type" not in node:
                        node["type"] = ""
                    if "mid" not in node:
                        node["mid"] = ""

                id_ += 1
                yield id_, sample



# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Medical Dialog dataset in english and chinese"""


import copy
import os
import re

import datasets


_CITATION = """\
@article{chen2020meddiag,
  title={MedDialog: a large-scale medical dialogue dataset},
  author={Chen, Shu and Ju, Zeqian and Dong, Xiangyu and Fang, Hongchao and Wang, Sicheng and Yang, Yue and Zeng, Jiaqi and Zhang, Ruisi and Zhang, Ruoyu and Zhou, Meng and Zhu, Penghui and Xie, Pengtao},
  journal={arXiv preprint arXiv:2004.03329},
  year={2020}
}
"""


_DESCRIPTION = """\
The MedDialog dataset (English) contains conversations (in English) between doctors and patients.\
It has 0.26 million dialogues. The data is continuously growing and more dialogues will be added. \
The raw dialogues are from healthcaremagic.com and icliniq.com.\

All copyrights of the data belong to healthcaremagic.com and icliniq.com.
"""

_HOMEPAGE = "https://github.com/UCSD-AI4H/Medical-Dialogue-System"

_LICENSE = ""


class MedicalDialog(datasets.GeneratorBasedBuilder):
    VERSION = datasets.Version("1.0.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="en", description="The dataset of medical dialogs in English.", version=VERSION),
        datasets.BuilderConfig(name="zh", description="The dataset of medical dialogs in Chinese.", version=VERSION),
    ]

    @property
    def manual_download_instructions(self):
        return """\
    \n  For English:\nYou need to go to https://drive.google.com/drive/folders/1g29ssimdZ6JzTST6Y8g6h-ogUNReBtJD?usp=sharing,\
    and manually download the dataset from Google Drive. Once it is completed,
    a file named Medical-Dialogue-Dataset-English-<timestamp-info>.zip will appear in your Downloads folder(
    or whichever folder your browser chooses to save files to). Unzip the folder to obtain
    a folder named "Medical-Dialogue-Dataset-English" several text files.

    Now, you can specify the path to this folder for the data_dir argument in the
    datasets.load_dataset(...) option.
    The <path/to/folder> can e.g. be "/Downloads/Medical-Dialogue-Dataset-English".
    The data can then be loaded using the below command:\
         `datasets.load_dataset("medical_dialog", name="en", data_dir="/Downloads/Medical-Dialogue-Dataset-English")`.

    \n  For Chinese:\nFollow the above process. Change the 'name' to 'zh'.The download link is https://drive.google.com/drive/folders/1r09_i8nJ9c1nliXVGXwSqRYqklcHd9e2

    **NOTE**
    - A caution while downloading from drive. It is better to download single files since creating a zip might not include files <500 MB. This has been observed mutiple times.
    - After downloading the files and adding them to the appropriate folder, the path of the folder can be given as input tu the data_dir path.
    """

    def _info(self):
        if self.config.name == "zh":
            features = datasets.Features(
                {
                    "file_name": datasets.Value("string"),
                    "dialogue_id": datasets.Value("int32"),
                    "dialogue_url": datasets.Value("string"),
                    "dialogue_turns": datasets.Sequence(
                        {
                            "speaker": datasets.ClassLabel(names=["病人", "医生"]),
                            "utterance": datasets.Value("string"),
                        }
                    ),
                }
            )

        if self.config.name == "en":
            features = datasets.Features(
                {
                    "file_name": datasets.Value("string"),
                    "dialogue_id": datasets.Value("int32"),
                    "dialogue_url": datasets.Value("string"),
                    "dialogue_turns": datasets.Sequence(
                        {
                            "speaker": datasets.ClassLabel(names=["Patient", "Doctor"]),
                            "utterance": datasets.Value("string"),
                        }
                    ),
                }
            )

        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        path_to_manual_file = os.path.abspath(os.path.expanduser(dl_manager.manual_dir))
        if not os.path.exists(path_to_manual_file):
            raise FileNotFoundError(
                f"{path_to_manual_file} does not exist. Make sure you insert a manual dir via `datasets.load_dataset('medical_dialog', data_dir=...)`. Manual download instructions: {self.manual_download_instructions})"
            )

        filepaths = [
            os.path.join(path_to_manual_file, txt_file_name)
            for txt_file_name in sorted(os.listdir(path_to_manual_file))
            if txt_file_name.endswith("txt")
        ]

        return [datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepaths": filepaths})]

    def _generate_examples(self, filepaths):
        """Yields examples. Iterates over each file and give the creates the corresponding features.

        NOTE:
        - The code makes some assumption on the structure of the raw .txt file.
        - There are some checks to separate different id's. Hopefully, should not cause further issues later when more txt files are added.
        """
        data_lang = self.config.name
        id_ = -1
        for filepath in filepaths:
            with open(filepath, encoding="utf-8") as f_in:
                # Parameters to just "sectionize" the raw data
                last_part = ""
                last_dialog = {}
                last_list = []
                last_user = ""
                check_list = []

                # These flags are present to have a single function address both chinese and english data
                # English data is a little hahazard (i.e. the sentences spans multiple different lines),
                # Chinese is compact with one line for doctor and patient.
                conv_flag = False
                des_flag = False

                while True:
                    line = f_in.readline()
                    if not line:
                        break

                    # Extracting the dialog id
                    if line[:2] == "id":  # Hardcode alert!
                        # Handling ID references that may come in the description
                        # These were observed in the Chinese dataset and were not
                        # followed by numbers
                        try:
                            dialogue_id = int(re.findall(r"\d+", line)[0])
                        except IndexError:
                            continue

                    # Extracting the url
                    if line[:4] == "http":  # Hardcode alert!
                        dialogue_url = line.rstrip()

                    # Extracting the patient info from description.
                    if line[:11] == "Description":  # Hardcode alert!
                        last_part = "description"
                        last_dialog = {}
                        last_list = []
                        last_user = ""
                        last_conv = {"speaker": "", "utterance": ""}
                        while True:
                            line = f_in.readline()
                            if (not line) or (line in ["\n", "\n\r"]):
                                break
                            else:
                                if data_lang == "zh":  # Condition in chinese
                                    if line[:5] == "病情描述：":  # Hardcode alert!
                                        last_user = "病人"
                                        sen = f_in.readline().rstrip()
                                        des_flag = True

                                if data_lang == "en":
                                    last_user = "Patient"
                                    sen = line.rstrip()
                                    des_flag = True

                                if des_flag:
                                    if sen == "":
                                        continue
                                    if sen in check_list:
                                        last_conv["speaker"] = ""
                                        last_conv["utterance"] = ""
                                    else:
                                        last_conv["speaker"] = last_user
                                        last_conv["utterance"] = sen
                                        check_list.append(sen)
                                    des_flag = False
                                    break
                    # Extracting the conversation info from dialogue.
                    elif line[:8] == "Dialogue":  # Hardcode alert!
                        if last_part == "description" and len(last_conv["utterance"]) > 0:
                            last_part = "dialogue"
                            if data_lang == "zh":
                                last_user = "病人"

                            if data_lang == "en":
                                last_user = "Patient"

                            while True:
                                line = f_in.readline()
                                if (not line) or (line in ["\n", "\n\r"]):
                                    conv_flag = False
                                    last_user = ""
                                    last_list.append(copy.deepcopy(last_conv))
                                    # To ensure close of conversation, only even number of sentences
                                    # are extracted
                                    last_turn = len(last_list)
                                    if int(last_turn / 2) > 0:
                                        temp = int(last_turn / 2)
                                        id_ += 1
                                        last_dialog["file_name"] = filepath
                                        last_dialog["dialogue_id"] = dialogue_id
                                        last_dialog["dialogue_url"] = dialogue_url
                                        last_dialog["dialogue_turns"] = last_list[: temp * 2]
                                        yield id_, last_dialog
                                    break

                                if data_lang == "zh":
                                    if line[:3] == "病人：" or line[:3] == "医生：":  # Hardcode alert!
                                        user = line[:2]  # Hardcode alert!
                                        line = f_in.readline()
                                        conv_flag = True

                                # The elif block is to ensure that multi-line sentences are captured.
                                # This has been observed only in english.
                                if data_lang == "en":
                                    if line.strip() == "Patient:" or line.strip() == "Doctor:":  # Hardcode alert!
                                        user = line.replace(":", "").rstrip()
                                        line = f_in.readline()
                                        conv_flag = True
                                    elif line[:2] != "id":  # Hardcode alert!
                                        conv_flag = True

                                # Continues till the next ID is parsed
                                if conv_flag:
                                    sen = line.rstrip()
                                    if sen == "":
                                        continue

                                    if user == last_user:
                                        last_conv["utterance"] = last_conv["utterance"] + sen
                                    else:
                                        last_user = user
                                        last_list.append(copy.deepcopy(last_conv))
                                        last_conv["utterance"] = sen
                                        last_conv["speaker"] = user
